from typing import Any
from typing import Optional

from ...sql.dml import Insert as StandardInsert
from ...sql.elements import ClauseElement

class Insert(StandardInsert):
    stringify_dialect: str = ...
    def excluded(self): ...
    def on_conflict_do_update(
        self,
        constraint: Optional[Any] = ...,
        index_elements: Optional[Any] = ...,
        index_where: Optional[Any] = ...,
        set_: Optional[Any] = ...,
        where: Optional[Any] = ...,
    ) -> None: ...
    def on_conflict_do_nothing(
        self,
        constraint: Optional[Any] = ...,
        index_elements: Optional[Any] = ...,
        index_where: Optional[Any] = ...,
    ) -> None: ...

insert: Any

class OnConflictClause(ClauseElement):
    stringify_dialect: str = ...
    constraint_target: Any = ...
    inferred_target_elements: Any = ...
    inferred_target_whereclause: Any = ...
    def __init__(
        self,
        constraint: Optional[Any] = ...,
        index_elements: Optional[Any] = ...,
        index_where: Optional[Any] = ...,
    ) -> None: ...

class OnConflictDoNothing(OnConflictClause):
    __visit_name__: str = ...

class OnConflictDoUpdate(OnConflictClause):
    __visit_name__: str = ...
    update_values_to_set: Any = ...
    update_whereclause: Any = ...
    def __init__(
        self,
        constraint: Optional[Any] = ...,
        index_elements: Optional[Any] = ...,
        index_where: Optional[Any] = ...,
        set_: Optional[Any] = ...,
        where: Optional[Any] = ...,
    ) -> None: ...



from typing import Any

from sqlalchemy.dialects.firebird.base import BIGINT as BIGINT
from sqlalchemy.dialects.firebird.base import BLOB as BLOB
from sqlalchemy.dialects.firebird.base import CHAR as CHAR
from sqlalchemy.dialects.firebird.base import DATE as DATE
from sqlalchemy.dialects.firebird.base import FLOAT as FLOAT
from sqlalchemy.dialects.firebird.base import NUMERIC as NUMERIC
from sqlalchemy.dialects.firebird.base import SMALLINT as SMALLINT
from sqlalchemy.dialects.firebird.base import TEXT as TEXT
from sqlalchemy.dialects.firebird.base import TIME as TIME
from sqlalchemy.dialects.firebird.base import TIMESTAMP as TIMESTAMP
from sqlalchemy.dialects.firebird.base import VARCHAR as VARCHAR

dialect: Any



from typing import Any

from .psycopg2 import PGDialect_psycopg2 as PGDialect_psycopg2

class PGDialect_psycopg2cffi(PGDialect_psycopg2):
    driver: str = ...
    supports_unicode_statements: bool = ...
    FEATURE_VERSION_MAP: Any = ...
    @classmethod
    def dbapi(cls): ...

dialect = PGDialect_psycopg2cffi



from typing import Any
from typing import Callable
from typing import Dict
from typing import Iterable
from typing import Iterator
from typing import List
from typing import Optional
from typing import TypeVar

from ..util import langhelpers

_ET = TypeVar("_ET", bound=ExternalTraversal)
_T = TypeVar("_T")

class TraversibleType(type):
    def __init__(cls, clsname: Any, bases: Any, clsdict: Any) -> None: ...

class Traversible(metaclass=TraversibleType): ...

class _InternalTraversalType(type):
    def __init__(cls, clsname: Any, bases: Any, clsdict: Any) -> None: ...

class InternalTraversal(object, metaclass=_InternalTraversalType):
    def dispatch(self, visit_symbol: Any) -> Optional[Callable[..., Any]]: ...
    def run_generated_dispatch(
        self,
        target: Any,
        internal_dispatch: Any,
        generate_dispatcher_name: Any,
    ) -> Any: ...
    def generate_dispatch(
        self,
        target_cls: Any,
        internal_dispatch: Any,
        generate_dispatcher_name: Any,
    ) -> Any: ...
    dp_has_cache_key: langhelpers._symbol = ...
    dp_has_cache_key_list: langhelpers._symbol = ...
    dp_clauseelement: langhelpers._symbol = ...
    dp_fromclause_canonical_column_collection: langhelpers._symbol = ...
    dp_clauseelement_tuples: langhelpers._symbol = ...
    dp_clauseelement_list: langhelpers._symbol = ...
    dp_clauseelement_tuple: langhelpers._symbol = ...
    dp_executable_options: langhelpers._symbol = ...
    dp_fromclause_ordered_set: langhelpers._symbol = ...
    dp_string: langhelpers._symbol = ...
    dp_string_list: langhelpers._symbol = ...
    dp_anon_name: langhelpers._symbol = ...
    dp_boolean: langhelpers._symbol = ...
    dp_operator: langhelpers._symbol = ...
    dp_type: langhelpers._symbol = ...
    dp_plain_dict: langhelpers._symbol = ...
    dp_dialect_options: langhelpers._symbol = ...
    dp_string_clauseelement_dict: langhelpers._symbol = ...
    dp_string_multi_dict: langhelpers._symbol = ...
    dp_annotations_key: langhelpers._symbol = ...
    dp_plain_obj: langhelpers._symbol = ...
    dp_named_ddl_element: langhelpers._symbol = ...
    dp_prefix_sequence: langhelpers._symbol = ...
    dp_table_hint_list: langhelpers._symbol = ...
    dp_setup_join_tuple: langhelpers._symbol = ...
    dp_statement_hint_list: langhelpers._symbol = ...
    dp_unknown_structure: langhelpers._symbol = ...
    dp_dml_ordered_values: langhelpers._symbol = ...
    dp_dml_values: langhelpers._symbol = ...
    dp_dml_multi_values: langhelpers._symbol = ...
    dp_propagate_attrs: langhelpers._symbol = ...

class ExtendedInternalTraversal(InternalTraversal):
    dp_ignore: langhelpers._symbol = ...
    dp_inspectable: langhelpers._symbol = ...
    dp_multi: langhelpers._symbol = ...
    dp_multi_list: langhelpers._symbol = ...
    dp_has_cache_key_tuples: langhelpers._symbol = ...
    dp_inspectable_list: langhelpers._symbol = ...

class ExternalTraversal:
    __traverse_options__: Any = ...
    def traverse_single(self, obj: Any, **kw: Any) -> Any: ...
    def iterate(self, obj: Any) -> Iterator[Any]: ...
    def traverse(self, obj: _T) -> _T: ...
    @property
    def visitor_iterator(self) -> Iterator[Any]: ...
    def chain(self: _ET, visitor: Any) -> _ET: ...

class CloningExternalTraversal(ExternalTraversal):
    def copy_and_process(self, list_: Iterable[_T]) -> List[_T]: ...
    def traverse(self, obj: _T) -> _T: ...

class ReplacingExternalTraversal(CloningExternalTraversal):
    def replace(self, elem: Any) -> Optional[Any]: ...
    def traverse(self, obj: _T) -> _T: ...

Visitable = Traversible
VisitableType = TraversibleType
ClauseVisitor = ExternalTraversal
CloningVisitor = CloningExternalTraversal
ReplacingCloningVisitor = ReplacingExternalTraversal

def iterate(obj: Any, opts: Dict[str, Any] = ...) -> Iterator[Any]: ...
def traverse_using(
    iterator: Iterator[Any], obj: _T, visitors: Dict[str, Callable[..., Any]]
) -> _T: ...
def traverse(
    obj: _T, opts: Dict[str, Any], visitors: Dict[str, Callable[..., Any]]
) -> _T: ...
def cloned_traverse(
    obj: _T, opts: Dict[str, Any], visitors: Dict[str, Callable[..., Any]]
) -> _T: ...
def replacement_traverse(
    obj: _T, opts: Dict[str, Any], replace: Callable[[Any], Any]
) -> _T: ...



from typing import Any

from .base import MySQLDialect as MySQLDialect
from .base import MySQLExecutionContext as MySQLExecutionContext
from .types import TIME as TIME
from ... import util as util
from ...connectors.pyodbc import PyODBCConnector as PyODBCConnector
from ...sql.sqltypes import Time as Time

class _pyodbcTIME(TIME):
    def result_processor(self, dialect: Any, coltype: Any): ...

class MySQLExecutionContext_pyodbc(MySQLExecutionContext):
    def get_lastrowid(self): ...

class MySQLDialect_pyodbc(PyODBCConnector, MySQLDialect):
    colspecs: Any = ...
    supports_unicode_statements: bool = ...
    execution_ctx_cls: Any = ...
    pyodbc_driver_name: str = ...
    def on_connect(self): ...

dialect = MySQLDialect_pyodbc



from typing import Any
from typing import Optional
from typing import Type
from typing import TypeVar

from .interfaces import LoaderOption
from ..sql.base import Generative

_L = TypeVar("_L", bound=Load)
_LO = TypeVar("_LO", bound=loader_option)

class Load(Generative, LoaderOption):
    path: Any = ...
    context: Any = ...
    local_opts: Any = ...
    is_class_strategy: bool = ...
    def __init__(self, entity: Any) -> None: ...
    @classmethod
    def for_existing_path(cls: Type[_L], path: Any) -> _L: ...
    is_opts_only: bool = ...
    strategy: Any = ...
    propagate_to_loaders: bool = ...
    def process_compile_state(self, compile_state: Any) -> None: ...
    def options(self: _L, *opts: Any) -> _L: ...
    def set_relationship_strategy(
        self: _L, attr: Any, strategy: Any, propagate_to_loaders: bool = ...
    ) -> _L: ...
    def set_column_strategy(
        self: _L,
        attrs: Any,
        strategy: Any,
        opts: Optional[Any] = ...,
        opts_only: bool = ...,
    ) -> _L: ...
    def set_generic_strategy(self: _L, attrs: Any, strategy: Any) -> _L: ...
    def set_class_strategy(self: _L, strategy: Any, opts: Any) -> _L: ...
    def contains_eager(
        loadopt: _L, attr: Any, alias: Optional[Any] = ...
    ) -> _L: ...
    def load_only(loadopt: _L, *attrs: Any) -> _L: ...
    def joinedload(
        loadopt: _L, attr: Any, innerjoin: Optional[bool] = ...
    ) -> _L: ...
    def subqueryload(loadopt: _L, attr: Any) -> _L: ...
    def selectinload(loadopt: _L, attr: Any) -> _L: ...
    def lazyload(loadopt: _L, attr: Any) -> _L: ...
    def immediateload(loadopt: _L, attr: Any) -> _L: ...
    def noload(loadopt: _L, attr: Any) -> _L: ...
    def raiseload(loadopt: _L, attr: Any, sql_only: bool = ...) -> _L: ...
    def defaultload(loadopt: _L, attr: Any) -> _L: ...
    def defer(loadopt: _L, key: str, raiseload: bool = ...) -> _L: ...
    def undefer(loadopt: _L, key: str) -> _L: ...
    def undefer_group(loadopt: _L, name: str) -> _L: ...
    def with_expression(loadopt: _L, key: Any, expression: Any) -> _L: ...
    def selectin_polymorphic(loadopt: _L, classes: Any) -> _L: ...

class _UnboundLoad(Load):
    path: Any = ...
    local_opts: Any = ...
    def __init__(self) -> None: ...

class loader_option:
    def __init__(self) -> None: ...
    name: Any = ...
    def __call__(self: _LO, fn: Any) -> _LO: ...

class _containseager_loader_option(loader_option):
    def fn(self, loadopt: _L, attr: Any, alias: Optional[Any] = ...) -> _L: ...
    def _unbound_fn(
        self, attr: Any, alias: Optional[Any] = ...
    ) -> _UnboundLoad: ...

class _load_only_loader_option(loader_option):
    def fn(self, loadopt: _L, *attrs: Any) -> _L: ...
    def _unbound_fn(self, *attrs: Any) -> _UnboundLoad: ...

class _joinedload_loader_option(loader_option):
    def fn(
        self, loadopt: _L, attr: Any, innerjoin: Optional[bool] = ...
    ) -> _L: ...
    def _unbound_fn(
        self, attr: Any, innerjoin: Optional[bool] = ...
    ) -> _UnboundLoad: ...

class _subqueryload_loader_option(loader_option):
    def fn(self, loadopt: _L, attr: Any) -> _L: ...
    def _unbound_fn(self, attr: Any) -> _UnboundLoad: ...

class _selectinload_loader_option(loader_option):
    def fn(self, loadopt: _L, attr: Any) -> _L: ...
    def _unbound_fn(self, attr: Any) -> _UnboundLoad: ...

class _lazyload_loader_option(loader_option):
    def fn(self, loadopt: _L, attr: Any) -> _L: ...
    def _unbound_fn(self, attr: Any) -> _UnboundLoad: ...

class _immediateload_loader_option(loader_option):
    def fn(self, loadopt: _L, attr: Any) -> _L: ...
    def _unbound_fn(self, attr: Any) -> _UnboundLoad: ...

class _noload_loader_option(loader_option):
    def fn(self, loadopt: _L, attr: Any) -> _L: ...
    def _unbound_fn(self, attr: Any) -> _UnboundLoad: ...

class _raiseload_loader_option(loader_option):
    def fn(self, loadopt: _L, attr: Any, sql_only: bool = ...) -> _L: ...
    def _unbound_fn(self, attr: Any, sql_only: bool = ...) -> _UnboundLoad: ...

class _defaultload_loader_option(loader_option):
    def fn(self, loadopt: _L, attr: Any) -> _L: ...
    def _unbound_fn(self, attr: Any) -> _UnboundLoad: ...

class _defer_loader_option(loader_option):
    def fn(self, loadopt: _L, key: str, raiseload: bool = ...) -> _L: ...
    def _unbound_fn(self, key: str, raiseload: bool = ...) -> _UnboundLoad: ...

class _undefer_loader_option(loader_option):
    def fn(self, loadopt: _L, key: str) -> _L: ...
    def _unbound_fn(self, key: str) -> _UnboundLoad: ...

class _undefer_group_loader_option(loader_option):
    def fn(self, loadopt: _L, name: str) -> _L: ...
    def _unbound_fn(self, name: str) -> _UnboundLoad: ...

class _with_expression_loader_option(loader_option):
    def fn(self, loadopt: _L, key: Any, expression: Any) -> _L: ...
    def _unbound_fn(
        self, loadopt: _L, key: Any, expression: Any
    ) -> _UnboundLoad: ...

class _selectin_polymorphic_loader_option(loader_option):
    def fn(self, loadopt: _L, classes: Any) -> _L: ...
    def _unbound_fn(self, classes: Any) -> _UnboundLoad: ...

contains_eager: _containseager_loader_option
load_only: _load_only_loader_option
joinedload: _joinedload_loader_option
subqueryload: _subqueryload_loader_option
selectinload: _selectinload_loader_option
lazyload: _lazyload_loader_option
immediateload: _immediateload_loader_option
noload: _noload_loader_option
raiseload: _raiseload_loader_option
defaultload: _defaultload_loader_option
defer: _defer_loader_option
undefer: _undefer_loader_option
undefer_group: _undefer_group_loader_option
with_expression: _with_expression_loader_option
selectin_polymorphic: _selectin_polymorphic_loader_option



from typing import Any
from typing import Optional

from .base import FBDialect as FBDialect
from .base import FBExecutionContext as FBExecutionContext
from ... import types as sqltypes
from ... import util as util

class _kinterbasdb_numeric:
    def bind_processor(self, dialect: Any): ...

class _FBNumeric_kinterbasdb(_kinterbasdb_numeric, sqltypes.Numeric): ...
class _FBFloat_kinterbasdb(_kinterbasdb_numeric, sqltypes.Float): ...

class FBExecutionContext_kinterbasdb(FBExecutionContext):
    @property
    def rowcount(self): ...

class FBDialect_kinterbasdb(FBDialect):
    driver: str = ...
    supports_sane_rowcount: bool = ...
    supports_sane_multi_rowcount: bool = ...
    execution_ctx_cls: Any = ...
    supports_native_decimal: bool = ...
    colspecs: Any = ...
    enable_rowcount: Any = ...
    type_conv: Any = ...
    concurrency_level: Any = ...
    retaining: Any = ...
    def __init__(
        self,
        type_conv: int = ...,
        concurrency_level: int = ...,
        enable_rowcount: bool = ...,
        retaining: bool = ...,
        **kwargs: Any,
    ) -> None: ...
    @classmethod
    def dbapi(cls): ...
    def do_execute(
        self,
        cursor: Any,
        statement: Any,
        parameters: Any,
        context: Optional[Any] = ...,
    ) -> None: ...
    def do_rollback(self, dbapi_connection: Any) -> None: ...
    def do_commit(self, dbapi_connection: Any) -> None: ...
    def create_connect_args(self, url: Any): ...
    def is_disconnect(self, e: Any, connection: Any, cursor: Any): ...

dialect = FBDialect_kinterbasdb



from typing import Any
from typing import Optional

from .json import JSON as JSON
from .json import JSONIndexType as JSONIndexType
from .json import JSONPathType as JSONPathType
from ... import exc as exc
from ... import processors as processors
from ... import sql as sql
from ... import types as sqltypes
from ... import util as util
from ...engine import default as default
from ...engine import reflection as reflection
from ...sql import coercions as coercions
from ...sql import ColumnElement as ColumnElement
from ...sql import compiler as compiler
from ...sql import elements as elements
from ...sql import roles as roles
from ...sql import schema as schema
from ...types import BLOB as BLOB
from ...types import BOOLEAN as BOOLEAN
from ...types import CHAR as CHAR
from ...types import DECIMAL as DECIMAL
from ...types import FLOAT as FLOAT
from ...types import INTEGER as INTEGER
from ...types import NUMERIC as NUMERIC
from ...types import REAL as REAL
from ...types import SMALLINT as SMALLINT
from ...types import TEXT as TEXT
from ...types import TIMESTAMP as TIMESTAMP
from ...types import VARCHAR as VARCHAR

class _SQliteJson(JSON):
    def result_processor(self, dialect: Any, coltype: Any): ...

class _DateTimeMixin:
    def __init__(
        self,
        storage_format: Optional[Any] = ...,
        regexp: Optional[Any] = ...,
        **kw: Any,
    ) -> None: ...
    @property
    def format_is_text_affinity(self): ...
    def adapt(self, cls: Any, **kw: Any): ...
    def literal_processor(self, dialect: Any): ...

class DATETIME(_DateTimeMixin, sqltypes.DateTime):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def bind_processor(self, dialect: Any): ...
    def result_processor(self, dialect: Any, coltype: Any): ...

class DATE(_DateTimeMixin, sqltypes.Date):
    def bind_processor(self, dialect: Any): ...
    def result_processor(self, dialect: Any, coltype: Any): ...

class TIME(_DateTimeMixin, sqltypes.Time):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def bind_processor(self, dialect: Any): ...
    def result_processor(self, dialect: Any, coltype: Any): ...

colspecs: Any
ischema_names: Any

class SQLiteCompiler(compiler.SQLCompiler):
    extract_map: Any = ...
    def visit_now_func(self, fn: Any, **kw: Any): ...
    def visit_localtimestamp_func(self, func: Any, **kw: Any): ...
    def visit_true(self, expr: Any, **kw: Any): ...
    def visit_false(self, expr: Any, **kw: Any): ...
    def visit_char_length_func(self, fn: Any, **kw: Any): ...
    def visit_cast(self, cast: Any, **kwargs: Any): ...
    def visit_extract(self, extract: Any, **kw: Any): ...
    def limit_clause(self, select: Any, **kw: Any): ...
    def for_update_clause(self, select: Any, **kw: Any): ...
    def visit_is_distinct_from_binary(
        self, binary: Any, operator: Any, **kw: Any
    ): ...
    def visit_is_not_distinct_from_binary(
        self, binary: Any, operator: Any, **kw: Any
    ): ...
    def visit_json_getitem_op_binary(
        self, binary: Any, operator: Any, **kw: Any
    ): ...
    def visit_json_path_getitem_op_binary(
        self, binary: Any, operator: Any, **kw: Any
    ): ...
    def visit_empty_set_expr(self, element_types: Any): ...
    def visit_regexp_match_op_binary(
        self, binary: Any, operator: Any, **kw: Any
    ): ...
    def visit_not_regexp_match_op_binary(
        self, binary: Any, operator: Any, **kw: Any
    ): ...
    def visit_on_conflict_do_nothing(self, on_conflict: Any, **kw: Any): ...
    def visit_on_conflict_do_update(self, on_conflict: Any, **kw: Any): ...

class SQLiteDDLCompiler(compiler.DDLCompiler):
    def get_column_specification(self, column: Any, **kwargs: Any): ...
    def visit_primary_key_constraint(self, constraint: Any): ...
    def visit_unique_constraint(self, constraint: Any): ...
    def visit_check_constraint(self, constraint: Any): ...
    def visit_column_check_constraint(self, constraint: Any): ...
    def visit_foreign_key_constraint(self, constraint: Any): ...
    def define_constraint_remote_table(
        self, constraint: Any, table: Any, preparer: Any
    ): ...
    def visit_create_index(
        self,
        create: Any,
        include_schema: bool = ...,
        include_table_schema: bool = ...,
    ): ...
    def post_create_table(self, table: Any): ...

class SQLiteTypeCompiler(compiler.GenericTypeCompiler):
    def visit_large_binary(self, type_: Any, **kw: Any): ...
    def visit_DATETIME(self, type_: Any, **kw: Any): ...
    def visit_DATE(self, type_: Any, **kw: Any): ...
    def visit_TIME(self, type_: Any, **kw: Any): ...
    def visit_JSON(self, type_: Any, **kw: Any): ...

class SQLiteIdentifierPreparer(compiler.IdentifierPreparer):
    reserved_words: Any = ...

class SQLiteExecutionContext(default.DefaultExecutionContext): ...

class SQLiteDialect(default.DefaultDialect):
    name: str = ...
    supports_alter: bool = ...
    supports_unicode_statements: bool = ...
    supports_unicode_binds: bool = ...
    supports_default_values: bool = ...
    supports_empty_insert: bool = ...
    supports_cast: bool = ...
    supports_multivalues_insert: bool = ...
    tuple_in_values: bool = ...
    default_paramstyle: str = ...
    execution_ctx_cls: Any = ...
    statement_compiler: Any = ...
    ddl_compiler: Any = ...
    type_compiler: Any = ...
    preparer: Any = ...
    ischema_names: Any = ...
    colspecs: Any = ...
    isolation_level: Any = ...
    construct_arguments: Any = ...
    native_datetime: Any = ...
    def __init__(
        self,
        isolation_level: Optional[Any] = ...,
        native_datetime: bool = ...,
        json_serializer: Optional[Any] = ...,
        json_deserializer: Optional[Any] = ...,
        _json_serializer: Optional[Any] = ...,
        _json_deserializer: Optional[Any] = ...,
        **kwargs: Any,
    ) -> None: ...
    def set_isolation_level(self, connection: Any, level: Any) -> None: ...
    def get_isolation_level(self, connection: Any): ...
    def on_connect(self): ...
    def get_schema_names(self, connection: Any, **kw: Any): ...
    def get_table_names(
        self, connection: Any, schema: Optional[Any] = ..., **kw: Any
    ): ...
    def get_temp_table_names(self, connection: Any, **kw: Any): ...
    def get_temp_view_names(self, connection: Any, **kw: Any): ...
    def has_table(
        self, connection: Any, table_name: Any, schema: Optional[Any] = ...
    ): ...
    def get_view_names(
        self, connection: Any, schema: Optional[Any] = ..., **kw: Any
    ): ...
    def get_view_definition(
        self,
        connection: Any,
        view_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_columns(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_pk_constraint(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_foreign_keys(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_unique_constraints(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_check_constraints(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_indexes(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...



from typing import Any
from typing import Optional

from .. import event as event
from .. import exc as exc
from .. import log as log
from .. import util as util

reset_rollback: Any
reset_commit: Any
reset_none: Any

class _ConnDialect:
    is_async: bool = ...
    def do_rollback(self, dbapi_connection: Any) -> None: ...
    def do_commit(self, dbapi_connection: Any) -> None: ...
    def do_close(self, dbapi_connection: Any) -> None: ...
    def do_ping(self, dbapi_connection: Any) -> None: ...

class Pool(log.Identified):
    logging_name: Any = ...
    echo: Any = ...
    def __init__(
        self,
        creator: Any,
        recycle: int = ...,
        echo: Optional[Any] = ...,
        logging_name: Optional[Any] = ...,
        reset_on_return: bool = ...,
        events: Optional[Any] = ...,
        dialect: Optional[Any] = ...,
        pre_ping: bool = ...,
        _dispatch: Optional[Any] = ...,
    ) -> None: ...
    def recreate(self) -> None: ...
    def dispose(self) -> None: ...
    def connect(self): ...
    def status(self) -> None: ...

class _ConnectionRecord:
    finalize_callback: Any = ...
    def __init__(self, pool: Any, connect: bool = ...) -> None: ...
    fresh: bool = ...
    fairy_ref: Any = ...
    starttime: Any = ...
    connection: Any = ...
    def info(self): ...
    def record_info(self): ...
    @classmethod
    def checkout(cls, pool: Any): ...
    def checkin(self, _fairy_was_created: bool = ...) -> None: ...
    @property
    def in_use(self): ...
    @property
    def last_connect_time(self): ...
    def close(self) -> None: ...
    def invalidate(self, e: Optional[Any] = ..., soft: bool = ...) -> None: ...
    def get_connection(self): ...

class _ConnectionFairy:
    connection: Any = ...
    def __init__(
        self, dbapi_connection: Any, connection_record: Any, echo: Any
    ) -> None: ...
    @property
    def is_valid(self): ...
    def info(self): ...
    @property
    def record_info(self): ...
    def invalidate(self, e: Optional[Any] = ..., soft: bool = ...) -> None: ...
    def cursor(self, *args: Any, **kwargs: Any): ...
    def __getattr__(self, key: Any): ...
    def detach(self) -> None: ...
    def close(self) -> None: ...



from typing import Any
from typing import Optional

from sqlalchemy import exc as exc
from sqlalchemy import types as sqltypes
from sqlalchemy import util as util
from sqlalchemy.engine import default as default
from sqlalchemy.engine import reflection as reflection
from sqlalchemy.sql import compiler as compiler
from sqlalchemy.sql import text as text
from sqlalchemy.types import BIGINT as BIGINT
from sqlalchemy.types import BINARY as BINARY
from sqlalchemy.types import CHAR as CHAR
from sqlalchemy.types import DATE as DATE
from sqlalchemy.types import DATETIME as DATETIME
from sqlalchemy.types import DECIMAL as DECIMAL
from sqlalchemy.types import FLOAT as FLOAT
from sqlalchemy.types import INT as INT
from sqlalchemy.types import INTEGER as INTEGER
from sqlalchemy.types import NCHAR as NCHAR
from sqlalchemy.types import NUMERIC as NUMERIC
from sqlalchemy.types import NVARCHAR as NVARCHAR
from sqlalchemy.types import REAL as REAL
from sqlalchemy.types import SMALLINT as SMALLINT
from sqlalchemy.types import TEXT as TEXT
from sqlalchemy.types import TIME as TIME
from sqlalchemy.types import TIMESTAMP as TIMESTAMP
from sqlalchemy.types import Unicode as Unicode
from sqlalchemy.types import VARBINARY as VARBINARY
from sqlalchemy.types import VARCHAR as VARCHAR

RESERVED_WORDS: Any

class _SybaseUnitypeMixin:
    def result_processor(self, dialect: Any, coltype: Any): ...

class UNICHAR(_SybaseUnitypeMixin, sqltypes.Unicode):
    __visit_name__: str = ...

class UNIVARCHAR(_SybaseUnitypeMixin, sqltypes.Unicode):
    __visit_name__: str = ...

class UNITEXT(_SybaseUnitypeMixin, sqltypes.UnicodeText):
    __visit_name__: str = ...

class TINYINT(sqltypes.Integer):
    __visit_name__: str = ...

class BIT(sqltypes.TypeEngine):
    __visit_name__: str = ...

class MONEY(sqltypes.TypeEngine):
    __visit_name__: str = ...

class SMALLMONEY(sqltypes.TypeEngine):
    __visit_name__: str = ...

class UNIQUEIDENTIFIER(sqltypes.TypeEngine):
    __visit_name__: str = ...

class IMAGE(sqltypes.LargeBinary):
    __visit_name__: str = ...

class SybaseTypeCompiler(compiler.GenericTypeCompiler):
    def visit_large_binary(self, type_: Any, **kw: Any): ...
    def visit_boolean(self, type_: Any, **kw: Any): ...
    def visit_unicode(self, type_: Any, **kw: Any): ...
    def visit_UNICHAR(self, type_: Any, **kw: Any): ...
    def visit_UNIVARCHAR(self, type_: Any, **kw: Any): ...
    def visit_UNITEXT(self, type_: Any, **kw: Any): ...
    def visit_TINYINT(self, type_: Any, **kw: Any): ...
    def visit_IMAGE(self, type_: Any, **kw: Any): ...
    def visit_BIT(self, type_: Any, **kw: Any): ...
    def visit_MONEY(self, type_: Any, **kw: Any): ...
    def visit_SMALLMONEY(self, type_: Any, **kw: Any): ...
    def visit_UNIQUEIDENTIFIER(self, type_: Any, **kw: Any): ...

ischema_names: Any

class SybaseInspector(reflection.Inspector):
    def __init__(self, conn: Any) -> None: ...
    def get_table_id(self, table_name: Any, schema: Optional[Any] = ...): ...

class SybaseExecutionContext(default.DefaultExecutionContext):
    def set_ddl_autocommit(self, connection: Any, value: Any) -> None: ...
    def pre_exec(self) -> None: ...
    def post_exec(self) -> None: ...
    def get_lastrowid(self): ...

class SybaseSQLCompiler(compiler.SQLCompiler):
    ansi_bind_rules: bool = ...
    extract_map: Any = ...
    def get_from_hint_text(self, table: Any, text: Any): ...
    def limit_clause(self, select: Any, **kw: Any): ...
    def visit_extract(self, extract: Any, **kw: Any): ...
    def visit_now_func(self, fn: Any, **kw: Any): ...
    def for_update_clause(self, select: Any): ...
    def order_by_clause(self, select: Any, **kw: Any): ...
    def delete_table_clause(
        self, delete_stmt: Any, from_table: Any, extra_froms: Any
    ): ...
    def delete_extra_from_clause(
        self,
        delete_stmt: Any,
        from_table: Any,
        extra_froms: Any,
        from_hints: Any,
        **kw: Any,
    ): ...

class SybaseDDLCompiler(compiler.DDLCompiler):
    def get_column_specification(self, column: Any, **kwargs: Any): ...
    def visit_drop_index(self, drop: Any): ...

class SybaseIdentifierPreparer(compiler.IdentifierPreparer):
    reserved_words: Any = ...

class SybaseDialect(default.DefaultDialect):
    name: str = ...
    supports_unicode_statements: bool = ...
    supports_sane_rowcount: bool = ...
    supports_sane_multi_rowcount: bool = ...
    supports_native_boolean: bool = ...
    supports_unicode_binds: bool = ...
    postfetch_lastrowid: bool = ...
    colspecs: Any = ...
    ischema_names: Any = ...
    type_compiler: Any = ...
    statement_compiler: Any = ...
    ddl_compiler: Any = ...
    preparer: Any = ...
    inspector: Any = ...
    construct_arguments: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    max_identifier_length: int = ...
    def initialize(self, connection: Any) -> None: ...
    def get_table_id(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_columns(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_foreign_keys(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_indexes(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_pk_constraint(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_schema_names(self, connection: Any, **kw: Any): ...
    def get_table_names(
        self, connection: Any, schema: Optional[Any] = ..., **kw: Any
    ): ...
    def get_view_definition(
        self,
        connection: Any,
        view_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_view_names(
        self, connection: Any, schema: Optional[Any] = ..., **kw: Any
    ): ...
    def has_table(
        self, connection: Any, table_name: Any, schema: Optional[Any] = ...
    ): ...



from typing import Any

from ...sql.dml import Insert as StandardInsert
from ...sql.elements import ClauseElement

class Insert(StandardInsert):
    stringify_dialect: str = ...
    @property
    def inserted(self): ...
    def inserted_alias(self): ...
    def on_duplicate_key_update(self, *args: Any, **kw: Any) -> None: ...

insert: Any

class OnDuplicateClause(ClauseElement):
    __visit_name__: str = ...
    stringify_dialect: str = ...
    inserted_alias: Any = ...
    update: Any = ...
    def __init__(self, inserted_alias: Any, update: Any) -> None: ...



# fmt: off
from ... import exc as exc
from ... import inspect as inspect
from ... import text as text
from ...testing.provision import create_db as create_db
from ...testing.provision import drop_all_schema_objects_post_tables as drop_all_schema_objects_post_tables
from ...testing.provision import drop_all_schema_objects_pre_tables as drop_all_schema_objects_pre_tables
from ...testing.provision import drop_db as drop_db
from ...testing.provision import log as log
from ...testing.provision import prepare_for_drop_tables as prepare_for_drop_tables
from ...testing.provision import set_default_schema_on_connection as set_default_schema_on_connection
from ...testing.provision import temp_table_keyword_args as temp_table_keyword_args



# fmt: off
from ... import exc as exc
from ...testing.provision import configure_follower as configure_follower
from ...testing.provision import create_db as create_db
from ...testing.provision import drop_db as drop_db
from ...testing.provision import generate_driver_url as generate_driver_url
from ...testing.provision import temp_table_keyword_args as temp_table_keyword_args



from typing import Any

from mypy.plugin import AttributeContext as AttributeContext
from mypy.plugin import Callable as Callable
from mypy.plugin import CallableType as CallableType
from mypy.plugin import ClassDefContext as ClassDefContext
from mypy.plugin import DynamicClassDefContext as DynamicClassDefContext
from mypy.plugin import FunctionContext as FunctionContext
from mypy.plugin import FunctionSigContext as FunctionSigContext
from mypy.plugin import List as List
from mypy.plugin import MethodContext as MethodContext
from mypy.plugin import MethodSigContext as MethodSigContext
from mypy.plugin import MypyFile as MypyFile
from mypy.plugin import Optional as Optional
from mypy.plugin import Plugin
from mypy.plugin import Tuple as Tuple
from mypy.plugin import Type as Type

class CustomPlugin(Plugin):
    def get_type_analyze_hook(self, fullname: str) -> Any: ...
    def get_additional_deps(
        self, file: MypyFile
    ) -> List[Tuple[int, str, int]]: ...
    def get_function_signature_hook(
        self, fullname: str
    ) -> Optional[Callable[[FunctionSigContext], CallableType]]: ...
    def get_function_hook(
        self, fullname: str
    ) -> Optional[Callable[[FunctionContext], Type]]: ...
    def get_method_signature_hook(
        self, fullname: str
    ) -> Optional[Callable[[MethodSigContext], CallableType]]: ...
    def get_method_hook(
        self, fullname: str
    ) -> Optional[Callable[[MethodContext], Type]]: ...
    def get_attribute_hook(
        self, fullname: str
    ) -> Optional[Callable[[AttributeContext], Type]]: ...
    def get_class_decorator_hook(
        self, fullname: str
    ) -> Optional[Callable[[ClassDefContext], None]]: ...
    def get_metaclass_hook(
        self, fullname: str
    ) -> Optional[Callable[[ClassDefContext], None]]: ...
    def get_base_class_hook(
        self, fullname: str
    ) -> Optional[Callable[[ClassDefContext], None]]: ...
    def get_customize_class_mro_hook(
        self, fullname: str
    ) -> Optional[Callable[[ClassDefContext], None]]: ...
    def get_dynamic_class_hook(
        self, fullname: str
    ) -> Optional[Callable[[DynamicClassDefContext], None]]: ...

def debug(ctx: Any) -> None: ...
def plugin(version: str) -> Any: ...



from typing import Any

from .base import BLOB as BLOB
from .base import BOOLEAN as BOOLEAN
from .base import CHAR as CHAR
from .base import DATE as DATE
from .base import DATETIME as DATETIME
from .base import DECIMAL as DECIMAL
from .base import FLOAT as FLOAT
from .base import INTEGER as INTEGER
from .base import JSON as JSON
from .base import NUMERIC as NUMERIC
from .base import REAL as REAL
from .base import SMALLINT as SMALLINT
from .base import TEXT as TEXT
from .base import TIME as TIME
from .base import TIMESTAMP as TIMESTAMP
from .base import VARCHAR as VARCHAR
from .dml import Insert as Insert
from .dml import insert as insert

dialect: Any



from typing import Any
from typing import NoReturn
from typing import TypeVar

from . import base
from . import interfaces
from ..sql.compiler import Compiled

_TMockConnection = TypeVar("_TMockConnection", bound=MockConnection)

class MockConnection(base.Connectable):
    def __init__(self, dialect: interfaces.Dialect, execute: Any) -> None: ...
    engine: base.Engine = ...
    dialect: interfaces.Dialect = ...
    name: str = ...
    def schema_for_object(self, obj: Any) -> str: ...
    def connect(self: _TMockConnection, **kwargs: Any) -> _TMockConnection: ...  # type: ignore[override]
    def execution_options(
        self: _TMockConnection, **opt: Any
    ) -> _TMockConnection: ...
    def compiler(
        self, statement: Any, parameters: Any, **kwargs: Any
    ) -> Compiled: ...
    def create(self, entity: Any, **kwargs: Any) -> None: ...
    def drop(self, entity: Any, **kwargs: Any) -> None: ...
    def execute(
        self, object_: Any, *multiparams: Any, **params: Any
    ) -> NoReturn: ...

def create_mock_engine(
    url: Any, executor: Any, **kw: Any
) -> MockConnection: ...



from typing import Any
from typing import overload
from typing import Type

from . import exc as exc
from . import util as util
from .engine.base import Connection
from .engine.base import Engine
from .engine.reflection import Inspector
from .orm import InstanceState
from .orm import Mapper

# this doesn't really seem to work...
@overload
def inspect(subject: Any, raiseerr: bool = ...) -> Any: ...
@overload
def inspect(subject: Engine, raiseerr: bool = ...) -> Inspector: ...
@overload
def inspect(subject: Connection, raiseerr: bool = ...) -> Inspector: ...
@overload
def inspect(subject: object, raiseerr: bool = ...) -> InstanceState: ...
@overload
def inspect(subject: Type, raiseerr: bool = ...) -> Mapper: ...



from typing import Any

from .base import BIT as BIT
from .base import MySQLDialect as MySQLDialect
from .mysqldb import MySQLDialect_mysqldb as MySQLDialect_mysqldb
from ... import util as util

class _cymysqlBIT(BIT):
    def result_processor(self, dialect: Any, coltype: Any): ...

class MySQLDialect_cymysql(MySQLDialect_mysqldb):
    driver: str = ...
    description_encoding: Any = ...
    supports_sane_rowcount: bool = ...
    supports_sane_multi_rowcount: bool = ...
    supports_unicode_statements: bool = ...
    colspecs: Any = ...
    @classmethod
    def dbapi(cls): ...
    def is_disconnect(self, e: Any, connection: Any, cursor: Any): ...

dialect = MySQLDialect_cymysql



from typing import Any
from typing import Optional

from ..ext.hybrid import hybrid_property

class index_property(hybrid_property):
    attr_name: Any = ...
    index: Any = ...
    default: Any = ...
    datatype: Any = ...
    onebased: Any = ...
    def __init__(
        self,
        attr_name: Any,
        index: Any,
        default: Any = ...,
        datatype: Optional[Any] = ...,
        mutable: bool = ...,
        onebased: bool = ...,
    ): ...
    def fget(self, instance: Any): ...
    def fset(self, instance: Any, value: Any) -> None: ...
    def fdel(self, instance: Any) -> None: ...
    def expr(self, model: Any): ...



from typing import Any

from . import attributes as attributes
from . import exc as exc
from . import sync as sync
from . import unitofwork as unitofwork
from .interfaces import MANYTOMANY as MANYTOMANY
from .interfaces import MANYTOONE as MANYTOONE
from .interfaces import ONETOMANY as ONETOMANY
from .. import sql as sql
from .. import util as util

class DependencyProcessor:
    prop: Any = ...
    cascade: Any = ...
    mapper: Any = ...
    parent: Any = ...
    secondary: Any = ...
    direction: Any = ...
    post_update: Any = ...
    passive_deletes: Any = ...
    passive_updates: Any = ...
    enable_typechecks: Any = ...
    sort_key: Any = ...
    key: Any = ...
    def __init__(self, prop: Any) -> None: ...
    @classmethod
    def from_relationship(cls, prop: Any): ...
    def hasparent(self, state: Any): ...
    def per_property_preprocessors(self, uow: Any) -> None: ...
    def per_property_flush_actions(self, uow: Any) -> None: ...
    def per_state_flush_actions(
        self, uow: Any, states: Any, isdelete: Any
    ) -> None: ...
    def presort_deletes(self, uowcommit: Any, states: Any): ...
    def presort_saves(self, uowcommit: Any, states: Any): ...
    def process_deletes(self, uowcommit: Any, states: Any) -> None: ...
    def process_saves(self, uowcommit: Any, states: Any) -> None: ...
    def prop_has_changes(self, uowcommit: Any, states: Any, isdelete: Any): ...

class OneToManyDP(DependencyProcessor):
    def per_property_dependencies(
        self,
        uow: Any,
        parent_saves: Any,
        child_saves: Any,
        parent_deletes: Any,
        child_deletes: Any,
        after_save: Any,
        before_delete: Any,
    ) -> None: ...
    def per_state_dependencies(
        self,
        uow: Any,
        save_parent: Any,
        delete_parent: Any,
        child_action: Any,
        after_save: Any,
        before_delete: Any,
        isdelete: Any,
        childisdelete: Any,
    ) -> None: ...
    def presort_deletes(self, uowcommit: Any, states: Any) -> None: ...
    def presort_saves(self, uowcommit: Any, states: Any) -> None: ...
    def process_deletes(self, uowcommit: Any, states: Any) -> None: ...
    def process_saves(self, uowcommit: Any, states: Any) -> None: ...

class ManyToOneDP(DependencyProcessor):
    def __init__(self, prop: Any) -> None: ...
    def per_property_dependencies(
        self,
        uow: Any,
        parent_saves: Any,
        child_saves: Any,
        parent_deletes: Any,
        child_deletes: Any,
        after_save: Any,
        before_delete: Any,
    ) -> None: ...
    def per_state_dependencies(
        self,
        uow: Any,
        save_parent: Any,
        delete_parent: Any,
        child_action: Any,
        after_save: Any,
        before_delete: Any,
        isdelete: Any,
        childisdelete: Any,
    ) -> None: ...
    def presort_deletes(self, uowcommit: Any, states: Any) -> None: ...
    def presort_saves(self, uowcommit: Any, states: Any) -> None: ...
    def process_deletes(self, uowcommit: Any, states: Any) -> None: ...
    def process_saves(self, uowcommit: Any, states: Any) -> None: ...

class DetectKeySwitch(DependencyProcessor):
    def per_property_preprocessors(self, uow: Any) -> None: ...
    def per_property_flush_actions(self, uow: Any) -> None: ...
    def per_state_flush_actions(
        self, uow: Any, states: Any, isdelete: Any
    ) -> None: ...
    def presort_deletes(self, uowcommit: Any, states: Any) -> None: ...
    def presort_saves(self, uow: Any, states: Any) -> None: ...
    def prop_has_changes(self, uow: Any, states: Any, isdelete: Any): ...
    def process_deletes(self, uowcommit: Any, states: Any) -> None: ...
    def process_saves(self, uowcommit: Any, states: Any) -> None: ...

class ManyToManyDP(DependencyProcessor):
    def per_property_dependencies(
        self,
        uow: Any,
        parent_saves: Any,
        child_saves: Any,
        parent_deletes: Any,
        child_deletes: Any,
        after_save: Any,
        before_delete: Any,
    ) -> None: ...
    def per_state_dependencies(
        self,
        uow: Any,
        save_parent: Any,
        delete_parent: Any,
        child_action: Any,
        after_save: Any,
        before_delete: Any,
        isdelete: Any,
        childisdelete: Any,
    ) -> None: ...
    def presort_deletes(self, uowcommit: Any, states: Any) -> None: ...
    def presort_saves(self, uowcommit: Any, states: Any) -> None: ...
    def process_deletes(self, uowcommit: Any, states: Any) -> None: ...
    def process_saves(self, uowcommit: Any, states: Any) -> None: ...



from typing import Any
from typing import Mapping
from typing import Optional
from typing import Tuple

from . import strategy_options
from .attributes import AttributeEvent as AttributeEvent
from .attributes import InstrumentedAttribute as InstrumentedAttribute
from .attributes import Mapped as Mapped
from .attributes import QueryableAttribute as QueryableAttribute
from .context import QueryContext as QueryContext
from .decl_api import as_declarative as as_declarative
from .decl_api import declarative_base as declarative_base
from .decl_api import declarative_mixin as declarative_mixin
from .decl_api import DeclarativeMeta as DeclarativeMeta
from .decl_api import declared_attr as declared_attr
from .decl_api import has_inherited_table as has_inherited_table
from .decl_api import registry as registry
from .decl_api import synonym_for as synonym_for
from .descriptor_props import CompositeProperty as CompositeProperty
from .descriptor_props import SynonymProperty as SynonymProperty
from .dynamic import AppenderQuery as AppenderQuery
from .identity import IdentityMap as IdentityMap
from .instrumentation import ClassManager as ClassManager
from .interfaces import EXT_CONTINUE as EXT_CONTINUE
from .interfaces import EXT_SKIP as EXT_SKIP
from .interfaces import EXT_STOP as EXT_STOP
from .interfaces import InspectionAttr as InspectionAttr
from .interfaces import InspectionAttrInfo as InspectionAttrInfo
from .interfaces import MANYTOMANY as MANYTOMANY
from .interfaces import MANYTOONE as MANYTOONE
from .interfaces import MapperProperty as MapperProperty
from .interfaces import NOT_EXTENSION as NOT_EXTENSION
from .interfaces import ONETOMANY as ONETOMANY
from .interfaces import PropComparator as PropComparator
from .loading import merge_frozen_result as merge_frozen_result
from .loading import merge_result as merge_result
from .mapper import class_mapper as class_mapper
from .mapper import configure_mappers as configure_mappers
from .mapper import Mapper as Mapper
from .mapper import reconstructor as reconstructor
from .mapper import validates as validates
from .properties import ColumnProperty as ColumnProperty
from .query import AliasOption as AliasOption
from .query import FromStatement as FromStatement
from .query import Query as Query
from .relationships import foreign as foreign
from .relationships import RelationshipProperty as RelationshipProperty
from .relationships import remote as remote
from .scoping import scoped_session as scoped_session
from .session import close_all_sessions as close_all_sessions
from .session import make_transient as make_transient
from .session import make_transient_to_detached as make_transient_to_detached
from .session import object_session as object_session
from .session import ORMExecuteState as ORMExecuteState
from .session import Session as Session
from .session import sessionmaker as sessionmaker
from .session import SessionTransaction as SessionTransaction
from .state import AttributeState as AttributeState
from .state import InstanceState as InstanceState
from .strategy_options import Load as Load
from .unitofwork import UOWTransaction as UOWTransaction
from .util import aliased as aliased
from .util import Bundle as Bundle
from .util import CascadeOptions as CascadeOptions
from .util import join as join
from .util import LoaderCriteriaOption as LoaderCriteriaOption
from .util import object_mapper as object_mapper
from .util import outerjoin as outerjoin
from .util import polymorphic_union as polymorphic_union
from .util import was_deleted as was_deleted
from .util import with_parent as with_parent
from .util import with_polymorphic as with_polymorphic

def create_session(bind: Optional[Any] = ..., **kwargs: Any) -> Session: ...

with_loader_criteria = LoaderCriteriaOption
relationship = RelationshipProperty

def relation(*arg: Any, **kw: Any) -> RelationshipProperty[Any]: ...
def dynamic_loader(argument: Any, **kw: Any) -> RelationshipProperty[Any]: ...

column_property = ColumnProperty
composite = CompositeProperty

_BackrefResult = Tuple[str, Mapping[str, Any]]

def backref(name: str, **kwargs: Any) -> _BackrefResult: ...
def deferred(*columns: Any, **kw: Any) -> ColumnProperty: ...
def query_expression(default_expr: Any = ...) -> ColumnProperty: ...

mapper = Mapper
synonym = SynonymProperty

def clear_mappers() -> None: ...

joinedload = strategy_options.joinedload._unbound_fn
contains_eager = strategy_options.contains_eager._unbound_fn
defer = strategy_options.defer._unbound_fn
undefer = strategy_options.undefer._unbound_fn
undefer_group = strategy_options.undefer_group._unbound_fn
with_expression = strategy_options.with_expression._unbound_fn
load_only = strategy_options.load_only._unbound_fn
lazyload = strategy_options.lazyload._unbound_fn
subqueryload = strategy_options.subqueryload._unbound_fn
selectinload = strategy_options.selectinload._unbound_fn
immediateload = strategy_options.immediateload._unbound_fn
noload = strategy_options.noload._unbound_fn
raiseload = strategy_options.raiseload._unbound_fn
defaultload = strategy_options.defaultload._unbound_fn
selectin_polymorphic = strategy_options.selectin_polymorphic._unbound_fn

eagerload = joinedload

contains_alias = AliasOption



from typing import Any

from sqlalchemy.orm import attributes as attributes
from sqlalchemy.orm import instrumentation as instrumentation
from . import clsregistry as clsregistry
from .attributes import QueryableAttribute as QueryableAttribute
from .base import InspectionAttr as InspectionAttr
from .descriptor_props import CompositeProperty as CompositeProperty
from .descriptor_props import SynonymProperty as SynonymProperty
from .interfaces import MapperProperty as MapperProperty
from .properties import ColumnProperty as ColumnProperty
from .util import class_mapper as class_mapper
from .. import event as event
from .. import exc as exc
from .. import util as util
from ..sql import expression as expression
from ..sql.schema import Column as Column
from ..sql.schema import Table as Table
from ..util import topological as topological

class _MapperConfig:
    @classmethod
    def setup_mapping(
        cls, registry: Any, cls_: Any, dict_: Any, table: Any, mapper_kw: Any
    ): ...
    cls: Any = ...
    classname: Any = ...
    properties: Any = ...
    declared_attr_reg: Any = ...
    def __init__(self, registry: Any, cls_: Any) -> None: ...
    def set_cls_attribute(self, attrname: Any, value: Any): ...

class _ImperativeMapperConfig(_MapperConfig):
    dict_: Any = ...
    local_table: Any = ...
    def __init__(
        self, registry: Any, cls_: Any, table: Any, mapper_kw: Any
    ) -> None: ...
    def map(self, mapper_kw: Any = ...): ...

class _ClassScanMapperConfig(_MapperConfig):
    dict_: Any = ...
    persist_selectable: Any = ...
    declared_columns: Any = ...
    column_copies: Any = ...
    def __init__(
        self, registry: Any, cls_: Any, dict_: Any, table: Any, mapper_kw: Any
    ) -> None: ...
    def map(self, mapper_kw: Any = ...): ...

class _DeferredMapperConfig(_ClassScanMapperConfig):
    @property
    def cls(self): ...
    @cls.setter
    def cls(self, class_: Any) -> None: ...
    @classmethod
    def has_cls(cls, class_: Any): ...
    @classmethod
    def raise_unmapped_for_cls(cls, class_: Any) -> None: ...
    @classmethod
    def config_for_cls(cls, class_: Any): ...
    @classmethod
    def classes_for_base(cls, base_cls: Any, sort: bool = ...): ...
    def map(self, mapper_kw: Any = ...): ...



from typing import Any

from .schema import Table

class ConventionDict:
    const: Any = ...
    table: Any = ...
    convention: Any = ...
    def __init__(self, const: Any, table: Table, convention: Any) -> None: ...
    def __getitem__(self, key: Any) -> Any: ...



# fmt: off
from typing import Any
from typing import Optional

from . import attributes as attributes
from . import instrumentation as instrumentation
from . import loading as loading
from . import properties as properties
from .base import class_mapper as class_mapper
from .base import state_str as state_str
from .interfaces import EXT_SKIP as EXT_SKIP
from .interfaces import InspectionAttr as InspectionAttr
from .interfaces import MapperProperty as MapperProperty
from .interfaces import ORMEntityColumnsClauseRole as ORMEntityColumnsClauseRole
from .interfaces import ORMFromClauseRole as ORMFromClauseRole
from .path_registry import PathRegistry as PathRegistry
from .. import event as event
from .. import inspection as inspection
from .. import log as log
from .. import schema as schema
from .. import sql as sql
from .. import util as util
from ..sql import base as sql_base
from ..sql import coercions as coercions
from ..sql import expression as expression
from ..sql import operators as operators
from ..sql import roles as roles
from ..sql import visitors as visitors
from ..sql.selectable import LABEL_STYLE_TABLENAME_PLUS_COL as LABEL_STYLE_TABLENAME_PLUS_COL
from ..util import HasMemoized as HasMemoized
# fmt: on

NO_ATTRIBUTE: Any

class Mapper(
    ORMFromClauseRole,
    ORMEntityColumnsClauseRole,
    sql_base.MemoizedHasCacheKey,
    InspectionAttr,
):
    class_: Any = ...
    class_manager: Any = ...
    non_primary: Any = ...
    always_refresh: Any = ...
    version_id_prop: Any = ...
    version_id_col: Any = ...
    version_id_generator: bool = ...
    concrete: Any = ...
    single: bool = ...
    inherits: Any = ...
    local_table: Any = ...
    inherit_condition: Any = ...
    inherit_foreign_keys: Any = ...
    batch: Any = ...
    eager_defaults: Any = ...
    column_prefix: Any = ...
    polymorphic_on: Any = ...
    validators: Any = ...
    passive_updates: Any = ...
    passive_deletes: Any = ...
    legacy_is_orphan: Any = ...
    allow_partial_pks: Any = ...
    confirm_deleted_rows: bool = ...
    polymorphic_load: Any = ...
    polymorphic_identity: Any = ...
    polymorphic_map: Any = ...
    include_properties: Any = ...
    exclude_properties: Any = ...
    def __init__(
        self,
        class_: Any,
        local_table: Optional[Any] = ...,
        properties: Optional[Any] = ...,
        primary_key: Optional[Any] = ...,
        non_primary: bool = ...,
        inherits: Optional[Any] = ...,
        inherit_condition: Optional[Any] = ...,
        inherit_foreign_keys: Optional[Any] = ...,
        always_refresh: bool = ...,
        version_id_col: Optional[Any] = ...,
        version_id_generator: Optional[Any] = ...,
        polymorphic_on: Optional[Any] = ...,
        _polymorphic_map: Optional[Any] = ...,
        polymorphic_identity: Optional[Any] = ...,
        concrete: bool = ...,
        with_polymorphic: Optional[Any] = ...,
        polymorphic_load: Optional[Any] = ...,
        allow_partial_pks: bool = ...,
        batch: bool = ...,
        column_prefix: Optional[Any] = ...,
        include_properties: Optional[Any] = ...,
        exclude_properties: Optional[Any] = ...,
        passive_updates: bool = ...,
        passive_deletes: bool = ...,
        confirm_deleted_rows: bool = ...,
        eager_defaults: bool = ...,
        legacy_is_orphan: bool = ...,
        _compiled_cache_size: int = ...,
    ): ...
    is_mapper: bool = ...
    represents_outer_join: bool = ...
    @property
    def mapper(self): ...
    @property
    def entity(self): ...
    persist_selectable: Any = ...
    configured: bool = ...
    tables: Any = ...
    primary_key: Any = ...
    base_mapper: Any = ...
    columns: Any = ...
    c: Any = ...
    @property
    def mapped_table(self): ...
    def add_properties(self, dict_of_properties: Any) -> None: ...
    def add_property(self, key: Any, prop: Any) -> None: ...
    def has_property(self, key: Any): ...
    def get_property(self, key: Any, _configure_mappers: bool = ...): ...
    def get_property_by_column(self, column: Any): ...
    @property
    def iterate_properties(self): ...
    with_polymorphic_mappers: Any = ...
    def __clause_element__(self): ...
    def select_identity_token(self): ...
    @property
    def selectable(self): ...
    def attrs(self): ...
    def all_orm_descriptors(self): ...
    def synonyms(self): ...
    @property
    def entity_namespace(self): ...
    def column_attrs(self): ...
    def relationships(self): ...
    def composites(self): ...
    def common_parent(self, other: Any): ...
    def is_sibling(self, other: Any): ...
    def isa(self, other: Any): ...
    def iterate_to_root(self) -> None: ...
    def self_and_descendants(self): ...
    def polymorphic_iterator(self): ...
    def primary_mapper(self): ...
    @property
    def primary_base_mapper(self): ...
    def identity_key_from_row(
        self,
        row: Any,
        identity_token: Optional[Any] = ...,
        adapter: Optional[Any] = ...,
    ): ...
    def identity_key_from_primary_key(
        self, primary_key: Any, identity_token: Optional[Any] = ...
    ): ...
    def identity_key_from_instance(self, instance: Any): ...
    def primary_key_from_instance(self, instance: Any): ...
    def cascade_iterator(
        self, type_: Any, state: Any, halt_on: Optional[Any] = ...
    ) -> None: ...

class _OptGetColumnsNotAvailable(Exception): ...

def configure_mappers() -> None: ...
def reconstructor(fn: Any): ...
def validates(*names: Any, **kw: Any): ...

class _ColumnMapping(dict):
    mapper: Any = ...
    def __init__(self, mapper: Any) -> None: ...
    def __missing__(self, column: Any) -> None: ...



from typing import Any
from typing import Optional

from .. import util as util
from ..engine import Connection as _LegacyConnection
from ..engine import Engine as _LegacyEngine
from ..engine.base import OptionEngineMixin as OptionEngineMixin

NO_OPTIONS: Any

def create_engine(*arg: Any, **kw: Any): ...

class Connection(_LegacyConnection):
    def begin(self): ...
    def begin_nested(self): ...
    def commit(self) -> None: ...
    def rollback(self) -> None: ...
    def close(self) -> None: ...
    def execute(
        self,
        statement: Any,
        parameters: Optional[Any] = ...,
        execution_options: Optional[Any] = ...,
    ): ...
    def scalar(
        self,
        statement: Any,
        parameters: Optional[Any] = ...,
        execution_options: Optional[Any] = ...,
    ): ...

class Engine(_LegacyEngine):
    transaction: Any = ...
    run_callable: Any = ...
    execute: Any = ...
    scalar: Any = ...
    table_names: Any = ...
    has_table: Any = ...
    class _trans_ctx:
        conn: Any = ...
        def __init__(self, conn: Any) -> None: ...
        transaction: Any = ...
        def __enter__(self): ...
        def __exit__(self, type_: Any, value: Any, traceback: Any) -> None: ...
    def begin(self): ...
    def connect(self): ...

class OptionEngine(OptionEngineMixin, Engine): ...



from typing import Any
from typing import Optional

from .. import util as util
from ..orm import strategy_options as strategy_options
from ..orm.query import Query as Query
from ..orm.session import Session as Session
from ..sql import func as func
from ..sql import literal_column as literal_column
from ..util import collections_abc as collections_abc

log: Any

class Bakery:
    cls: Any = ...
    cache: Any = ...
    def __init__(self, cls_: Any, cache: Any) -> None: ...
    def __call__(self, initial_fn: Any, *args: Any): ...

class BakedQuery:
    steps: Any = ...
    def __init__(
        self, bakery: Any, initial_fn: Any, args: Any = ...
    ) -> None: ...
    @classmethod
    def bakery(cls, size: int = ..., _size_alert: Optional[Any] = ...): ...
    def __iadd__(self, other: Any): ...
    def __add__(self, other: Any): ...
    def add_criteria(self, fn: Any, *args: Any): ...
    def with_criteria(self, fn: Any, *args: Any): ...
    def for_session(self, session: Any): ...
    def __call__(self, session: Any): ...
    def spoil(self, full: bool = ...): ...
    def to_query(self, query_or_session: Any): ...

class Result:
    bq: Any = ...
    session: Any = ...
    def __init__(self, bq: Any, session: Any) -> None: ...
    def params(self, *args: Any, **kw: Any): ...
    def with_post_criteria(self, fn: Any): ...
    def __iter__(self) -> Any: ...
    def count(self): ...
    def scalar(self): ...
    def first(self): ...
    def one(self): ...
    def one_or_none(self): ...
    def all(self): ...
    def get(self, ident: Any): ...

def bake_lazy_loaders() -> None: ...
def unbake_lazy_loaders() -> None: ...
def baked_lazyload_all(*keys: Any): ...

baked_lazyload: Any
baked_lazyload_all: Any
bakery: Any



# fmt: off
from collections import defaultdict as defaultdict
from contextlib import contextmanager as contextmanager
from functools import partial as partial
from functools import update_wrapper as update_wrapper

from ._collections import coerce_generator_arg as coerce_generator_arg
from ._collections import coerce_to_immutabledict as coerce_to_immutabledict
from ._collections import collections_abc as collections_abc
from ._collections import column_dict as column_dict
from ._collections import column_set as column_set
from ._collections import EMPTY_DICT as EMPTY_DICT
from ._collections import EMPTY_SET as EMPTY_SET
from ._collections import FacadeDict as FacadeDict
from ._collections import flatten_iterator as flatten_iterator
from ._collections import has_dupes as has_dupes
from ._collections import has_intersection as has_intersection
from ._collections import IdentitySet as IdentitySet
from ._collections import ImmutableContainer as ImmutableContainer
from ._collections import immutabledict as immutabledict
from ._collections import ImmutableProperties as ImmutableProperties
from ._collections import LRUCache as LRUCache
from ._collections import ordered_column_set as ordered_column_set
from ._collections import OrderedDict as OrderedDict
from ._collections import OrderedIdentitySet as OrderedIdentitySet
from ._collections import OrderedProperties as OrderedProperties
from ._collections import OrderedSet as OrderedSet
from ._collections import PopulateDict as PopulateDict
from ._collections import Properties as Properties
from ._collections import ScopedRegistry as ScopedRegistry
from ._collections import sort_dictionary as sort_dictionary
from ._collections import ThreadLocalRegistry as ThreadLocalRegistry
from ._collections import to_column_set as to_column_set
from ._collections import to_list as to_list
from ._collections import to_set as to_set
from ._collections import unique_list as unique_list
from ._collections import UniqueAppender as UniqueAppender
from ._collections import update_copy as update_copy
from ._collections import WeakPopulateDict as WeakPopulateDict
from ._collections import WeakSequence as WeakSequence
from ._preloaded import preload_module as preload_module
from ._preloaded import preloaded as preloaded
from .compat import _SortKeyFunction as _SortKeyFunction
from .compat import ABC as ABC
from .compat import arm as arm
from .compat import b as b
from .compat import b64decode as b64decode
from .compat import b64encode as b64encode
from .compat import binary_type as binary_type
from .compat import byte_buffer as byte_buffer
from .compat import callable as callable
from .compat import cmp as cmp
from .compat import cpython as cpython
from .compat import dataclass_fields as dataclass_fields
from .compat import decode_backslashreplace as decode_backslashreplace
from .compat import dottedgetter as dottedgetter
from .compat import has_refcount_gc as has_refcount_gc
from .compat import inspect_getfullargspec as inspect_getfullargspec
from .compat import int_types as int_types
from .compat import iterbytes as iterbytes
from .compat import itertools_filter as itertools_filter
from .compat import itertools_filterfalse as itertools_filterfalse
from .compat import local_dataclass_fields as local_dataclass_fields
from .compat import namedtuple as namedtuple
from .compat import next as next
from .compat import nullcontext as nullcontext
from .compat import osx as osx
from .compat import parse_qsl as parse_qsl
from .compat import perf_counter as perf_counter
from .compat import pickle as pickle
from .compat import print_ as print_
from .compat import py2k as py2k
from .compat import py37 as py37
from .compat import py3k as py3k
from .compat import pypy as pypy
from .compat import quote_plus as quote_plus
from .compat import raise_ as raise_
from .compat import raise_from_cause as raise_from_cause
from .compat import reduce as reduce
from .compat import reraise as reraise
from .compat import string_types as string_types
from .compat import StringIO as StringIO
from .compat import text_type as text_type
from .compat import threading as threading
from .compat import timezone as timezone
from .compat import TYPE_CHECKING as TYPE_CHECKING
from .compat import u as u
from .compat import ue as ue
from .compat import unquote as unquote
from .compat import unquote_plus as unquote_plus
from .compat import win32 as win32
from .compat import with_metaclass as with_metaclass
from .compat import zip_longest as zip_longest
from .concurrency import asyncio as asyncio
from .concurrency import await_fallback as await_fallback
from .concurrency import await_only as await_only
from .concurrency import greenlet_spawn as greenlet_spawn
from .deprecations import deprecated as deprecated
from .deprecations import deprecated_20 as deprecated_20
from .deprecations import deprecated_20_cls as deprecated_20_cls
from .deprecations import deprecated_cls as deprecated_cls
from .deprecations import deprecated_params as deprecated_params
from .deprecations import inject_docstring_text as inject_docstring_text
from .deprecations import moved_20 as moved_20
from .deprecations import SQLALCHEMY_WARN_20 as SQLALCHEMY_WARN_20
from .deprecations import warn_deprecated as warn_deprecated
from .deprecations import warn_deprecated_20 as warn_deprecated_20
from .langhelpers import add_parameter_text as add_parameter_text
from .langhelpers import as_interface as as_interface
from .langhelpers import asbool as asbool
from .langhelpers import asint as asint
from .langhelpers import assert_arg_type as assert_arg_type
from .langhelpers import attrsetter as attrsetter
from .langhelpers import bool_or_str as bool_or_str
from .langhelpers import chop_traceback as chop_traceback
from .langhelpers import class_hierarchy as class_hierarchy
from .langhelpers import classproperty as classproperty
from .langhelpers import clsname_as_plain_name as clsname_as_plain_name
from .langhelpers import coerce_kw_type as coerce_kw_type
from .langhelpers import constructor_copy as constructor_copy
from .langhelpers import constructor_key as constructor_key
from .langhelpers import counter as counter
from .langhelpers import create_proxy_methods as create_proxy_methods
from .langhelpers import decode_slice as decode_slice
from .langhelpers import decorator as decorator
from .langhelpers import dictlike_iteritems as dictlike_iteritems
from .langhelpers import duck_type_collection as duck_type_collection
from .langhelpers import ellipses_string as ellipses_string
from .langhelpers import EnsureKWArgType as EnsureKWArgType
from .langhelpers import format_argspec_init as format_argspec_init
from .langhelpers import format_argspec_plus as format_argspec_plus
from .langhelpers import generic_repr as generic_repr
from .langhelpers import get_callable_argspec as get_callable_argspec
from .langhelpers import get_cls_kwargs as get_cls_kwargs
from .langhelpers import get_func_kwargs as get_func_kwargs
from .langhelpers import getargspec_init as getargspec_init
from .langhelpers import HasMemoized as HasMemoized
from .langhelpers import hybridmethod as hybridmethod
from .langhelpers import hybridproperty as hybridproperty
from .langhelpers import iterate_attributes as iterate_attributes
from .langhelpers import map_bits as map_bits
from .langhelpers import md5_hex as md5_hex
from .langhelpers import memoized_instancemethod as memoized_instancemethod
from .langhelpers import memoized_property as memoized_property
from .langhelpers import MemoizedSlots as MemoizedSlots
from .langhelpers import method_is_overridden as method_is_overridden
from .langhelpers import methods_equivalent as methods_equivalent
from .langhelpers import monkeypatch_proxied_specials as monkeypatch_proxied_specials
from .langhelpers import NoneType as NoneType
from .langhelpers import only_once as only_once
from .langhelpers import PluginLoader as PluginLoader
from .langhelpers import portable_instancemethod as portable_instancemethod
from .langhelpers import quoted_token_parser as quoted_token_parser
from .langhelpers import safe_reraise as safe_reraise
from .langhelpers import set_creation_order as set_creation_order
from .langhelpers import string_or_unprintable as string_or_unprintable
from .langhelpers import symbol as symbol
from .langhelpers import unbound_method_to_callable as unbound_method_to_callable
from .langhelpers import walk_subclasses as walk_subclasses
from .langhelpers import warn as warn
from .langhelpers import warn_exception as warn_exception
from .langhelpers import warn_limited as warn_limited
from .langhelpers import wrap_callable as wrap_callable



# fmt: off
from typing import Any

from .descriptor_props import CompositeProperty as CompositeProperty
from .descriptor_props import ConcreteInheritedProperty as ConcreteInheritedProperty
from .descriptor_props import SynonymProperty as SynonymProperty
from .interfaces import PropComparator
from .interfaces import StrategizedProperty
from .relationships import RelationshipProperty as RelationshipProperty
from .. import util
# fmt: on

class ColumnProperty(StrategizedProperty):
    strategy_wildcard_key: str = ...
    inherit_cache: bool = ...
    columns: Any = ...
    group: Any = ...
    deferred: Any = ...
    raiseload: Any = ...
    instrument: Any = ...
    comparator_factory: Any = ...
    descriptor: Any = ...
    active_history: Any = ...
    expire_on_flush: Any = ...
    info: Any = ...
    doc: Any = ...
    strategy_key: Any = ...
    def __init__(self, *columns: Any, **kwargs: Any) -> None: ...
    def __clause_element__(self): ...
    @property
    def expression(self): ...
    def instrument_class(self, mapper: Any) -> None: ...
    def do_init(self) -> None: ...
    def copy(self): ...
    def merge(
        self,
        session: Any,
        source_state: Any,
        source_dict: Any,
        dest_state: Any,
        dest_dict: Any,
        load: Any,
        _recursive: Any,
        _resolve_conflict_map: Any,
    ) -> None: ...
    class Comparator(util.MemoizedSlots, PropComparator):
        def _memoized_method___clause_element__(self): ...
        def operate(self, op: Any, *other: Any, **kwargs: Any): ...
        def reverse_operate(self, op: Any, other: Any, **kwargs: Any): ...



from typing import Any

from .enumerated import ENUM as ENUM
from .enumerated import SET as SET
from .types import DATETIME as DATETIME
from .types import TIME as TIME
from .types import TIMESTAMP as TIMESTAMP
from ... import log as log
from ... import util as util

class ReflectedState:
    columns: Any = ...
    table_options: Any = ...
    table_name: Any = ...
    keys: Any = ...
    fk_constraints: Any = ...
    ck_constraints: Any = ...
    def __init__(self) -> None: ...

class MySQLTableDefinitionParser:
    dialect: Any = ...
    preparer: Any = ...
    def __init__(self, dialect: Any, preparer: Any) -> None: ...
    def parse(self, show_create: Any, charset: Any): ...



from typing import Any

from .. import exc as exc
from .. import util as util

CANCEL: Any
NO_RETVAL: Any

def listen(
    target: Any, identifier: Any, fn: Any, *args: Any, **kw: Any
) -> None: ...
def listens_for(target: Any, identifier: Any, *args: Any, **kw: Any): ...
def remove(target: Any, identifier: Any, fn: Any) -> None: ...
def contains(target: Any, identifier: Any, fn: Any): ...



from typing import Any
from typing import Optional

from . import attributes as attributes
from .. import event as event
from .. import util as util
from ..util import topological as topological

def track_cascade_events(descriptor: Any, prop: Any): ...

class UOWTransaction:
    session: Any = ...
    attributes: Any = ...
    deps: Any = ...
    mappers: Any = ...
    presort_actions: Any = ...
    postsort_actions: Any = ...
    dependencies: Any = ...
    states: Any = ...
    post_update_states: Any = ...
    def __init__(self, session: Any): ...
    @property
    def has_work(self): ...
    def was_already_deleted(self, state: Any): ...
    def is_deleted(self, state: Any): ...
    def memo(self, key: Any, callable_: Any): ...
    def remove_state_actions(self, state: Any) -> None: ...
    def get_attribute_history(
        self, state: Any, key: Any, passive: Any = ...
    ): ...
    def has_dep(self, processor: Any): ...
    def register_preprocessor(
        self, processor: Any, fromparent: Any
    ) -> None: ...
    def register_object(
        self,
        state: Any,
        isdelete: bool = ...,
        listonly: bool = ...,
        cancel_delete: bool = ...,
        operation: Optional[Any] = ...,
        prop: Optional[Any] = ...,
    ): ...
    def register_post_update(
        self, state: Any, post_update_cols: Any
    ) -> None: ...
    def filter_states_for_dep(self, dep: Any, states: Any): ...
    def states_for_mapper_hierarchy(
        self, mapper: Any, isdelete: Any, listonly: Any
    ) -> None: ...
    def execute(self): ...
    def finalize_flush_changes(self) -> None: ...

class IterateMappersMixin: ...

class Preprocess(IterateMappersMixin):
    dependency_processor: Any = ...
    fromparent: Any = ...
    processed: Any = ...
    setup_flush_actions: bool = ...
    def __init__(self, dependency_processor: Any, fromparent: Any) -> None: ...
    def execute(self, uow: Any): ...

class PostSortRec:
    def __new__(cls, uow: Any, *args: Any): ...
    def execute_aggregate(self, uow: Any, recs: Any) -> None: ...

class ProcessAll(IterateMappersMixin, PostSortRec):
    dependency_processor: Any = ...
    sort_key: Any = ...
    isdelete: Any = ...
    fromparent: Any = ...
    def __init__(
        self,
        uow: Any,
        dependency_processor: Any,
        isdelete: Any,
        fromparent: Any,
    ) -> None: ...
    def execute(self, uow: Any) -> None: ...
    def per_state_flush_actions(self, uow: Any): ...

class PostUpdateAll(PostSortRec):
    mapper: Any = ...
    isdelete: Any = ...
    sort_key: Any = ...
    def __init__(self, uow: Any, mapper: Any, isdelete: Any) -> None: ...
    def execute(self, uow: Any) -> None: ...

class SaveUpdateAll(PostSortRec):
    mapper: Any = ...
    sort_key: Any = ...
    def __init__(self, uow: Any, mapper: Any) -> None: ...
    def execute(self, uow: Any) -> None: ...
    def per_state_flush_actions(self, uow: Any) -> None: ...

class DeleteAll(PostSortRec):
    mapper: Any = ...
    sort_key: Any = ...
    def __init__(self, uow: Any, mapper: Any) -> None: ...
    def execute(self, uow: Any) -> None: ...
    def per_state_flush_actions(self, uow: Any) -> None: ...

class ProcessState(PostSortRec):
    dependency_processor: Any = ...
    sort_key: Any = ...
    isdelete: Any = ...
    state: Any = ...
    def __init__(
        self, uow: Any, dependency_processor: Any, isdelete: Any, state: Any
    ) -> None: ...
    def execute_aggregate(self, uow: Any, recs: Any) -> None: ...

class SaveUpdateState(PostSortRec):
    state: Any = ...
    mapper: Any = ...
    sort_key: Any = ...
    def __init__(self, uow: Any, state: Any) -> None: ...
    def execute_aggregate(self, uow: Any, recs: Any) -> None: ...

class DeleteState(PostSortRec):
    state: Any = ...
    mapper: Any = ...
    sort_key: Any = ...
    def __init__(self, uow: Any, state: Any) -> None: ...
    def execute_aggregate(self, uow: Any, recs: Any) -> None: ...



from typing import Any
from typing import Callable
from typing import Set
from typing import TypeVar

_F = TypeVar("_F", bound=Callable[..., Any])

class _ModuleRegistry:
    module_registry: Set[str] = ...
    prefix: str = ...
    def __init__(self, prefix: str = ...) -> None: ...
    def preload_module(self, *deps: str) -> Callable[[_F], _F]: ...
    def import_prefix(self, path: Any) -> None: ...

preloaded: _ModuleRegistry

def preload_module(*deps: str) -> Callable[[_F], _F]: ...



from typing import Any
from typing import Optional

from .base import class_mapper as class_mapper
from .. import exc as exc
from .. import inspection as inspection
from .. import util as util
from ..sql import visitors as visitors
from ..sql.traversals import HasCacheKey as HasCacheKey

log: Any

class PathRegistry(HasCacheKey):
    is_token: bool = ...
    is_root: bool = ...
    def __eq__(self, other: Any) -> Any: ...
    def __ne__(self, other: Any) -> Any: ...
    def set(self, attributes: Any, key: Any, value: Any) -> None: ...
    def setdefault(self, attributes: Any, key: Any, value: Any) -> None: ...
    def get(self, attributes: Any, key: Any, value: Optional[Any] = ...): ...
    def __len__(self): ...
    def __hash__(self) -> Any: ...
    @property
    def length(self): ...
    def pairs(self) -> None: ...
    def contains_mapper(self, mapper: Any): ...
    def contains(self, attributes: Any, key: Any): ...
    def __reduce__(self): ...
    @classmethod
    def serialize_context_dict(cls, dict_: Any, tokens: Any): ...
    @classmethod
    def deserialize_context_dict(cls, serialized: Any): ...
    def serialize(self): ...
    @classmethod
    def deserialize(cls, path: Any): ...
    @classmethod
    def per_mapper(cls, mapper: Any): ...
    @classmethod
    def coerce(cls, raw: Any): ...
    def token(self, token: Any): ...
    def __add__(self, other: Any): ...

class RootRegistry(PathRegistry):
    inherit_cache: bool = ...
    path: Any = ...
    natural_path: Any = ...
    has_entity: bool = ...
    is_aliased_class: bool = ...
    is_root: bool = ...
    def __getitem__(self, entity: Any): ...

class PathToken(HasCacheKey, str):
    @classmethod
    def intern(cls, strvalue: Any): ...

class TokenRegistry(PathRegistry):
    inherit_cache: bool = ...
    token: Any = ...
    parent: Any = ...
    path: Any = ...
    natural_path: Any = ...
    def __init__(self, parent: Any, token: Any) -> None: ...
    has_entity: bool = ...
    is_token: bool = ...
    def generate_for_superclasses(self) -> None: ...
    def __getitem__(self, entity: Any) -> None: ...

class PropRegistry(PathRegistry):
    is_unnatural: bool = ...
    inherit_cache: bool = ...
    prop: Any = ...
    parent: Any = ...
    path: Any = ...
    natural_path: Any = ...
    def __init__(self, parent: Any, prop: Any) -> None: ...
    def has_entity(self): ...
    def entity(self): ...
    @property
    def mapper(self): ...
    @property
    def entity_path(self): ...
    def __getitem__(self, entity: Any): ...

class AbstractEntityRegistry(PathRegistry):
    has_entity: bool = ...
    key: Any = ...
    parent: Any = ...
    is_aliased_class: Any = ...
    entity: Any = ...
    path: Any = ...
    natural_path: Any = ...
    def __init__(self, parent: Any, entity: Any) -> None: ...
    @property
    def entity_path(self): ...
    @property
    def mapper(self): ...
    def __bool__(self): ...
    __nonzero__: Any = ...
    def __getitem__(self, entity: Any): ...

class SlotsEntityRegistry(AbstractEntityRegistry):
    inherit_cache: bool = ...

class CachingEntityRegistry(AbstractEntityRegistry, dict):
    inherit_cache: bool = ...
    def __getitem__(self, entity: Any): ...
    def __missing__(self, key: Any): ...



from typing import Any
from typing import Optional

from sqlalchemy import exc as exc
from sqlalchemy import sql as sql
from sqlalchemy import types as sqltypes
from sqlalchemy import util as util
from sqlalchemy.engine import default as default
from sqlalchemy.engine import reflection as reflection
from sqlalchemy.sql import compiler as compiler
from sqlalchemy.sql import expression as expression
from sqlalchemy.types import BIGINT as BIGINT
from sqlalchemy.types import BLOB as BLOB
from sqlalchemy.types import DATE as DATE
from sqlalchemy.types import FLOAT as FLOAT
from sqlalchemy.types import INTEGER as INTEGER
from sqlalchemy.types import Integer as Integer
from sqlalchemy.types import NUMERIC as NUMERIC
from sqlalchemy.types import SMALLINT as SMALLINT
from sqlalchemy.types import TEXT as TEXT
from sqlalchemy.types import TIME as TIME
from sqlalchemy.types import TIMESTAMP as TIMESTAMP

RESERVED_WORDS: Any

class _StringType(sqltypes.String):
    charset: Any = ...
    def __init__(self, charset: Optional[Any] = ..., **kw: Any) -> None: ...

class VARCHAR(_StringType, sqltypes.VARCHAR):
    __visit_name__: str = ...
    def __init__(self, length: Optional[Any] = ..., **kwargs: Any) -> None: ...

class CHAR(_StringType, sqltypes.CHAR):
    __visit_name__: str = ...
    def __init__(self, length: Optional[Any] = ..., **kwargs: Any) -> None: ...

class _FBDateTime(sqltypes.DateTime):
    def bind_processor(self, dialect: Any): ...

colspecs: Any
ischema_names: Any

class FBTypeCompiler(compiler.GenericTypeCompiler):
    def visit_boolean(self, type_: Any, **kw: Any): ...
    def visit_datetime(self, type_: Any, **kw: Any): ...
    def visit_TEXT(self, type_: Any, **kw: Any): ...
    def visit_BLOB(self, type_: Any, **kw: Any): ...
    def visit_CHAR(self, type_: Any, **kw: Any): ...
    def visit_VARCHAR(self, type_: Any, **kw: Any): ...

class FBCompiler(sql.compiler.SQLCompiler):
    ansi_bind_rules: bool = ...
    def visit_now_func(self, fn: Any, **kw: Any): ...
    def visit_startswith_op_binary(
        self, binary: Any, operator: Any, **kw: Any
    ): ...
    def visit_not_startswith_op_binary(
        self, binary: Any, operator: Any, **kw: Any
    ): ...
    def visit_mod_binary(self, binary: Any, operator: Any, **kw: Any): ...
    def visit_alias(self, alias: Any, asfrom: bool = ..., **kwargs: Any): ...
    def visit_substring_func(self, func: Any, **kw: Any): ...
    def visit_length_func(self, function: Any, **kw: Any): ...
    visit_char_length_func: Any = ...
    def function_argspec(self, func: Any, **kw: Any): ...
    def default_from(self): ...
    def visit_sequence(self, seq: Any, **kw: Any): ...
    def get_select_precolumns(self, select: Any, **kw: Any): ...
    def limit_clause(self, select: Any, **kw: Any): ...
    def returning_clause(self, stmt: Any, returning_cols: Any): ...

class FBDDLCompiler(sql.compiler.DDLCompiler):
    def visit_create_sequence(self, create: Any): ...
    def visit_drop_sequence(self, drop: Any): ...
    def visit_computed_column(self, generated: Any): ...

class FBIdentifierPreparer(sql.compiler.IdentifierPreparer):
    reserved_words: Any = ...
    illegal_initial_characters: Any = ...
    def __init__(self, dialect: Any) -> None: ...

class FBExecutionContext(default.DefaultExecutionContext):
    def fire_sequence(self, seq: Any, type_: Any): ...

class FBDialect(default.DefaultDialect):
    name: str = ...
    max_identifier_length: int = ...
    supports_sequences: bool = ...
    sequences_optional: bool = ...
    supports_default_values: bool = ...
    postfetch_lastrowid: bool = ...
    supports_native_boolean: bool = ...
    requires_name_normalize: bool = ...
    supports_empty_insert: bool = ...
    statement_compiler: Any = ...
    ddl_compiler: Any = ...
    preparer: Any = ...
    type_compiler: Any = ...
    execution_ctx_cls: Any = ...
    colspecs: Any = ...
    ischema_names: Any = ...
    construct_arguments: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    implicit_returning: Any = ...
    def initialize(self, connection: Any) -> None: ...
    def has_table(
        self, connection: Any, table_name: Any, schema: Optional[Any] = ...
    ): ...
    def has_sequence(
        self, connection: Any, sequence_name: Any, schema: Optional[Any] = ...
    ): ...
    def get_table_names(
        self, connection: Any, schema: Optional[Any] = ..., **kw: Any
    ): ...
    def get_view_names(
        self, connection: Any, schema: Optional[Any] = ..., **kw: Any
    ): ...
    def get_view_definition(
        self,
        connection: Any,
        view_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_pk_constraint(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_column_sequence(
        self,
        connection: Any,
        table_name: Any,
        column_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_columns(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_foreign_keys(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...
    def get_indexes(
        self,
        connection: Any,
        table_name: Any,
        schema: Optional[Any] = ...,
        **kw: Any,
    ): ...



from typing import Any
from typing import Optional

def Serializer(*args: Any, **kw: Any): ...
def Deserializer(
    file: Any,
    metadata: Optional[Any] = ...,
    scoped_session: Optional[Any] = ...,
    engine: Optional[Any] = ...,
): ...
def dumps(obj: Any, protocol: Any = ...): ...
def loads(
    data: Any,
    metadata: Optional[Any] = ...,
    scoped_session: Optional[Any] = ...,
    engine: Optional[Any] = ...,
): ...



import abc
from typing import Any
from typing import Optional

from .result import Result as Result
from .result import ResultMetaData as ResultMetaData
from .result import SimpleResultMetaData as SimpleResultMetaData
from .result import tuplegetter as tuplegetter
from .row import LegacyRow as LegacyRow
from .. import exc as exc
from .. import util as util
from ..sql import expression as expression
from ..sql import sqltypes as sqltypes
from ..sql.compiler import RM_NAME as RM_NAME
from ..sql.compiler import RM_OBJECTS as RM_OBJECTS
from ..sql.compiler import RM_RENDERED_NAME as RM_RENDERED_NAME
from ..sql.compiler import RM_TYPE as RM_TYPE

MD_INDEX: int
MD_RESULT_MAP_INDEX: int
MD_OBJECTS: int
MD_LOOKUP_KEY: int
MD_RENDERED_NAME: int
MD_PROCESSOR: int
MD_UNTRANSLATED: int

class CursorResultMetaData(ResultMetaData):
    returns_rows: bool = ...
    case_sensitive: Any = ...
    def __init__(self, parent: Any, cursor_description: Any) -> None: ...

class LegacyCursorResultMetaData(CursorResultMetaData): ...

class ResultFetchStrategy:
    alternate_cursor_description: Any = ...
    def soft_close(self, result: Any, dbapi_cursor: Any) -> None: ...
    def hard_close(self, result: Any, dbapi_cursor: Any) -> None: ...
    def yield_per(self, result: Any, dbapi_cursor: Any, num: Any) -> None: ...
    def fetchone(
        self, result: Any, dbapi_cursor: Any, hard_close: bool = ...
    ) -> None: ...
    def fetchmany(
        self, result: Any, dbapi_cursor: Any, size: Optional[Any] = ...
    ) -> None: ...
    def fetchall(self, result: Any) -> None: ...
    def handle_exception(
        self, result: Any, dbapi_cursor: Any, err: Any
    ) -> None: ...

class NoCursorFetchStrategy(ResultFetchStrategy):
    def soft_close(self, result: Any, dbapi_cursor: Any) -> None: ...
    def hard_close(self, result: Any, dbapi_cursor: Any) -> None: ...
    def fetchone(
        self, result: Any, dbapi_cursor: Any, hard_close: bool = ...
    ): ...
    def fetchmany(
        self, result: Any, dbapi_cursor: Any, size: Optional[Any] = ...
    ): ...
    def fetchall(self, result: Any, dbapi_cursor: Any): ...

class NoCursorDQLFetchStrategy(NoCursorFetchStrategy): ...
class NoCursorDMLFetchStrategy(NoCursorFetchStrategy): ...

class CursorFetchStrategy(ResultFetchStrategy):
    def soft_close(self, result: Any, dbapi_cursor: Any) -> None: ...
    def hard_close(self, result: Any, dbapi_cursor: Any) -> None: ...
    def handle_exception(
        self, result: Any, dbapi_cursor: Any, err: Any
    ) -> None: ...
    def yield_per(self, result: Any, dbapi_cursor: Any, num: Any) -> None: ...
    def fetchone(
        self, result: Any, dbapi_cursor: Any, hard_close: bool = ...
    ): ...
    def fetchmany(
        self, result: Any, dbapi_cursor: Any, size: Optional[Any] = ...
    ): ...
    def fetchall(self, result: Any, dbapi_cursor: Any): ...

class BufferedRowCursorFetchStrategy(CursorFetchStrategy):
    def __init__(
        self,
        dbapi_cursor: Any,
        execution_options: Any,
        growth_factor: int = ...,
        initial_buffer: Optional[Any] = ...,
    ) -> None: ...
    @classmethod
    def create(cls, result: Any): ...
    def yield_per(self, result: Any, dbapi_cursor: Any, num: Any) -> None: ...
    def soft_close(self, result: Any, dbapi_cursor: Any) -> None: ...
    def hard_close(self, result: Any, dbapi_cursor: Any) -> None: ...
    def fetchone(
        self, result: Any, dbapi_cursor: Any, hard_close: bool = ...
    ): ...
    def fetchmany(
        self, result: Any, dbapi_cursor: Any, size: Optional[Any] = ...
    ): ...
    def fetchall(self, result: Any, dbapi_cursor: Any): ...

class FullyBufferedCursorFetchStrategy(CursorFetchStrategy):
    alternate_cursor_description: Any = ...
    def __init__(
        self,
        dbapi_cursor: Any,
        alternate_description: Optional[Any] = ...,
        initial_buffer: Optional[Any] = ...,
    ) -> None: ...
    def yield_per(self, result: Any, dbapi_cursor: Any, num: Any) -> None: ...
    def soft_close(self, result: Any, dbapi_cursor: Any) -> None: ...
    def hard_close(self, result: Any, dbapi_cursor: Any) -> None: ...
    def fetchone(
        self, result: Any, dbapi_cursor: Any, hard_close: bool = ...
    ): ...
    def fetchmany(
        self, result: Any, dbapi_cursor: Any, size: Optional[Any] = ...
    ): ...
    def fetchall(self, result: Any, dbapi_cursor: Any): ...

class _NoResultMetaData(ResultMetaData):
    returns_rows: bool = ...
    @property
    def keys(self) -> None: ...

class BaseCursorResult:
    out_parameters: Any = ...
    closed: bool = ...
    context: Any = ...
    dialect: Any = ...
    cursor: Any = ...
    cursor_strategy: Any = ...
    connection: Any = ...
    def __init__(
        self, context: Any, cursor_strategy: Any, cursor_description: Any
    ): ...
    @property
    def inserted_primary_key_rows(self): ...
    @property
    def inserted_primary_key(self): ...
    def last_updated_params(self): ...
    def last_inserted_params(self): ...
    @property
    def returned_defaults_rows(self): ...
    @property
    def returned_defaults(self): ...
    def lastrow_has_defaults(self): ...
    def postfetch_cols(self): ...
    def prefetch_cols(self): ...
    def supports_sane_rowcount(self): ...
    def supports_sane_multi_rowcount(self): ...
    def rowcount(self): ...
    @property
    def lastrowid(self): ...
    @property
    def returns_rows(self): ...
    @property
    def is_insert(self): ...

class CursorResult(BaseCursorResult, Result):
    def merge(self, *others: Any): ...
    def close(self) -> None: ...
    def yield_per(self, num: Any) -> None: ...

class LegacyCursorResult(CursorResult):
    def close(self) -> None: ...

ResultProxy = LegacyCursorResult

class BufferedRowResultProxy(ResultProxy): ...
class FullyBufferedResultProxy(ResultProxy): ...
class BufferedColumnRow(LegacyRow, metaclass=abc.ABCMeta): ...
class BufferedColumnResultProxy(ResultProxy): ...



# fmt: off
from collections import namedtuple
from typing import Any
from typing import Optional

from . import attributes as attributes
from . import interfaces as interfaces
from . import loading as loading
from . import properties as properties
from . import query as query
from . import relationships as relationships
from . import unitofwork as unitofwork
from .context import ORMCompileState as ORMCompileState
from .context import QueryContext as QueryContext
from .interfaces import LoaderStrategy as LoaderStrategy
from .interfaces import StrategizedProperty as StrategizedProperty
from .state import InstanceState as InstanceState
from .util import aliased as aliased
from .. import event as event
from .. import inspect as inspect
from .. import log as log
from .. import sql as sql
from .. import util as util
from ..sql import visitors as visitors
from ..sql.selectable import LABEL_STYLE_TABLENAME_PLUS_COL as LABEL_STYLE_TABLENAME_PLUS_COL
# fmt: on

class UninstrumentedColumnLoader(LoaderStrategy):
    columns: Any = ...
    def __init__(self, parent: Any, strategy_key: Any) -> None: ...
    def setup_query(
        self,
        compile_state: Any,
        query_entity: Any,
        path: Any,
        loadopt: Any,
        adapter: Any,
        column_collection: Optional[Any] = ...,
        **kwargs: Any,
    ) -> None: ...
    def create_row_processor(
        self,
        context: Any,
        query_entity: Any,
        path: Any,
        loadopt: Any,
        mapper: Any,
        result: Any,
        adapter: Any,
        populators: Any,
    ) -> None: ...

class ColumnLoader(LoaderStrategy):
    columns: Any = ...
    is_composite: Any = ...
    def __init__(self, parent: Any, strategy_key: Any) -> None: ...
    def setup_query(
        self,
        compile_state: Any,
        query_entity: Any,
        path: Any,
        loadopt: Any,
        adapter: Any,
        column_collection: Any,
        memoized_populators: Any,
        check_for_adapt: bool = ...,
        **kwargs: Any,
    ) -> None: ...
    is_class_level: bool = ...
    def init_class_attribute(self, mapper: Any) -> None: ...
    def create_row_processor(
        self,
        context: Any,
        query_entity: Any,
        path: Any,
        loadopt: Any,
        mapper: Any,
        result: Any,
        adapter: Any,
        populators: Any,
    ) -> None: ...

class ExpressionColumnLoader(ColumnLoader):
    def __init__(self, parent: Any, strategy_key: Any) -> None: ...
    def setup_query(
        self,
        compile_state: Any,
        query_entity: Any,
        path: Any,
        loadopt: Any,
        adapter: Any,
        column_collection: Any,
        memoized_populators: Any,
        **kwargs: Any,
    ) -> None: ...
    def create_row_processor(
        self,
        context: Any,
        query_entity: Any,
        path: Any,
        loadopt: Any,
        mapper: Any,
        result: Any,
        adapter: Any,
        populators: Any,
    ) -> None: ...
    is_class_level: bool = ...
    def init_class_attribute(self, mapper: Any) -> None: ...

class DeferredColumnLoader(LoaderStrategy):
    raiseload: Any = ...
    columns: Any = ...
    group: Any = ...
    def __init__(self, parent: Any, strategy_key: Any) -> None: ...
    def create_row_processor(
        self,
        context: Any,
        query_entity: Any,
        path: Any,
        loadopt: Any,
        mapper: Any,
        result: Any,
        adapter: Any,
        populators: Any,
    ) -> None: ...
    is_class_level: bool = ...
    def init_class_attribute(self, mapper: Any) -> None: ...
    def setup_query(
        self,
        compile_state: Any,
        query_entity: Any,
        path: Any,
        loadopt: Any,
        adapter: Any,
        column_collection: Any,
        memoized_populators: Any,
        only_load_props: Optional[Any] = ...,
        **kw: Any,
    ) -> None: ...

class LoadDeferredColumns:
    key: Any = ...
    raiseload: Any = ...
    def __init__(self, key: Any, raiseload: bool = ...) -> None: ...
    def __call__(self, state: Any, passive: Any = ...): ...

class AbstractRelationshipLoader(LoaderStrategy):
    mapper: Any = ...
    entity: Any = ...
    target: Any = ...
    uselist: Any = ...
    def __init__(self, parent: Any, strategy_key: Any) -> None: ...

class DoNothingLoader(LoaderStrategy): ...

class NoLoader(AbstractRelationshipLoader):
    is_class_level: bool = ...
    def init_class_attribute(self, mapper: Any) -> None: ...
    def create_row_processor(
        self,
        context: Any,
        query_entity: Any,
        path: Any,
        loadopt: Any,
        mapper: Any,
        result: Any,
        adapter: Any,
        populators: Any,
    ) -> None: ...

class LazyLoader(AbstractRelationshipLoader, util.MemoizedSlots):
    is_aliased_class: Any = ...
    use_get: Any = ...
    def __init__(self, parent: Any, strategy_key: Any) -> None: ...
    is_class_level: bool = ...
    def init_class_attribute(self, mapper: Any) -> None: ...
    def create_row_processor(
        self,
        context: Any,
        query_entity: Any,
        path: Any,
        loadopt: Any,
        mapper: Any,
        result: Any,
        adapter: Any,
        populators: Any,
    ) -> None: ...

class LoadLazyAttribute:
    key: Any = ...
    strategy_key: Any = ...
    loadopt: Any = ...
    def __init__(
        self, key: Any, initiating_strategy: Any, loadopt: Any
    ) -> None: ...
    def __call__(self, state: Any, passive: Any = ...): ...

class PostLoader(AbstractRelationshipLoader): ...

class ImmediateLoader(PostLoader):
    def init_class_attribute(self, mapper: Any) -> None: ...
    def create_row_processor(
        self,
        context: Any,
        query_entity: Any,
        path: Any,
        loadopt: Any,
        mapper: Any,
        result: Any,
        adapter: Any,
        populators: Any,
    ) -> None: ...

class SubqueryLoader(PostLoader):
    join_depth: Any = ...
    def __init__(self, parent: Any, strategy_key: Any) -> None: ...
    def init_class_attribute(self, mapper: Any) -> None: ...
    class _SubqCollections:
        session: Any = ...
        execution_options: Any = ...
        load_options: Any = ...
        params: Any = ...
        subq: Any = ...
        def __init__(self, context: Any, subq: Any) -> None: ...
        def get(self, key: Any, default: Any): ...
        def loader(self, state: Any, dict_: Any, row: Any) -> None: ...
    def create_row_processor(
        self,
        context: Any,
        query_entity: Any,
        path: Any,
        loadopt: Any,
        mapper: Any,
        result: Any,
        adapter: Any,
        populators: Any,
    ): ...

class JoinedLoader(AbstractRelationshipLoader):
    join_depth: Any = ...
    def __init__(self, parent: Any, strategy_key: Any) -> None: ...
    def init_class_attribute(self, mapper: Any) -> None: ...
    def setup_query(
        self,
        compile_state: Any,
        query_entity: Any,
        path: Any,
        loadopt: Any,
        adapter: Any,
        column_collection: Optional[Any] = ...,
        parentmapper: Optional[Any] = ...,
        chained_from_outerjoin: bool = ...,
        **kwargs: Any,
    ) -> None: ...
    def create_row_processor(
        self,
        context: Any,
        query_entity: Any,
        path: Any,
        loadopt: Any,
        mapper: Any,
        result: Any,
        adapter: Any,
        populators: Any,
    ) -> None: ...

class SelectInLoader(PostLoader, util.MemoizedSlots):

    query_info = namedtuple(
        "queryinfo",
        [
            "load_only_child",
            "load_with_join",
            "in_expr",
            "pk_cols",
            "zero_idx",
            "child_lookup_cols",
        ],
    )
    join_depth: Any = ...
    omit_join: Any = ...
    def __init__(self, parent: Any, strategy_key: Any) -> None: ...
    def init_class_attribute(self, mapper: Any) -> None: ...
    def create_row_processor(
        self,
        context: Any,
        query_entity: Any,
        path: Any,
        loadopt: Any,
        mapper: Any,
        result: Any,
        adapter: Any,
        populators: Any,
    ): ...

def single_parent_validator(desc: Any, prop: Any): ...



from typing import Any
from typing import Callable
from typing import FrozenSet
from typing import Generic
from typing import MutableMapping
from typing import Optional
from typing import Tuple
from typing import Type
from typing import TypeVar
from typing import Union

from . import attributes as attributes
from . import clsregistry as clsregistry
from . import instrumentation as instrumentation
from . import interfaces as interfaces
from .mapper import Mapper as Mapper
from .. import inspection as inspection
from .. import util as util
from ..sql import ColumnElement
from ..sql import FromClause
from ..sql.schema import MetaData as MetaData
from ..util import hybridmethod as hybridmethod
from ..util import hybridproperty as hybridproperty

_T = TypeVar("_T")

def has_inherited_table(cls: type) -> bool: ...
def synonym_for(name: Any, map_column: bool = ...): ...

class declared_attr(interfaces._MappedAttribute, property, Generic[_T]):
    __doc__: Any = ...
    def __init__(self, fget: Any, cascading: bool = ...) -> None: ...
    def __get__(
        desc: Any, self: Any, cls: Any
    ) -> Union[interfaces.MapperProperty, ColumnElement]: ...
    def cascading(cls): ...

class _stateful_declared_attr(declared_attr):
    kw: Any = ...
    def __init__(self, **kw: Any) -> None: ...
    def __call__(self, fn: Any): ...

def declarative_mixin(cls: type) -> type: ...
def declarative_base(
    bind: Optional[
        Any
    ] = ...,  # NOTE: Deprecated in 1.4, to be removed in 2.0.
    metadata: Optional[MetaData] = ...,
    mapper: Optional[Callable[..., Mapper]] = ...,
    cls: Union[type, Tuple[type, ...]] = ...,
    name: str = ...,
    constructor: Callable[..., None] = ...,
    class_registry: Optional[MutableMapping[Any, Any]] = ...,
    metaclass: type = ...,
) -> type: ...

class registry:
    metadata: MetaData
    constructor: Callable[..., None]
    def __init__(
        self,
        metadata: Optional[MetaData] = ...,
        class_registry: Optional[MutableMapping[Any, Any]] = ...,
        constructor: Callable[..., None] = ...,
        _bind: Optional[Any] = ...,
    ) -> None: ...
    @property
    def mappers(self) -> FrozenSet[Mapper]: ...
    def configure(self, cascade: bool = ...) -> None: ...
    def dispose(self, cascade: bool = ...) -> None: ...
    def generate_base(
        self,
        mapper: Optional[Callable[..., Mapper]] = ...,
        cls: Union[type, Tuple[type, ...]] = ...,
        name: str = ...,
        metaclass: type = ...,
    ) -> type: ...
    def mapped(self, cls: Type[_T]) -> Type[_T]: ...
    def as_declarative_base(
        self, **kw: Any
    ) -> Callable[[Type[_T]], Type[_T]]: ...
    def map_declaratively(self, cls: type) -> Mapper: ...
    def map_imperatively(
        self,
        class_: type,
        local_table: Optional[FromClause] = ...,
        **kw: Any,
    ) -> Mapper: ...

_registry = registry

class DeclarativeMeta(type):
    def __init__(
        cls, classname: Any, bases: Any, dict_: Any, **kw: Any
    ) -> None: ...
    def __setattr__(cls, key: Any, value: Any) -> None: ...
    def __delattr__(cls, key: Any) -> None: ...
    metadata: MetaData
    registry: _registry  # Avoid circural reference

def as_declarative(**kw: Any) -> Callable[[Type[_T]], Type[_T]]: ...



from typing import Any

from ... import cast as cast
from ... import Column as Column
from ... import MetaData as MetaData
from ... import Table as Table
from ... import util as util
from ...ext.compiler import compiles as compiles
from ...sql import expression as expression
from ...types import Boolean as Boolean
from ...types import Integer as Integer
from ...types import Numeric as Numeric
from ...types import String as String
from ...types import TypeDecorator as TypeDecorator
from ...types import Unicode as Unicode

ischema: Any

class CoerceUnicode(TypeDecorator):
    impl: Any = ...
    def process_bind_param(self, value: Any, dialect: Any): ...
    def bind_expression(self, bindvalue: Any): ...

class _cast_on_2005(expression.ColumnElement):
    bindvalue: Any = ...
    def __init__(self, bindvalue: Any) -> None: ...

schemata: Any
tables: Any
columns: Any
mssql_temp_table_columns: Any
constraints: Any
column_constraints: Any
key_constraints: Any
ref_constraints: Any
views: Any
computed_columns: Any
sequences: Any

class IdentitySqlVariant(TypeDecorator):
    impl: Any = ...
    def column_expression(self, colexpr: Any): ...

identity_columns: Any



from typing import Any
from typing import Optional

from .. import exc as exc
from .. import inspect as inspect
from .. import orm as orm
from .. import util as util
from ..orm import collections as collections
from ..orm import interfaces as interfaces
from ..sql import or_ as or_
from ..sql.operators import ColumnOperators as ColumnOperators

def association_proxy(target_collection: Any, attr: Any, **kw: Any): ...

ASSOCIATION_PROXY: Any

class AssociationProxy(interfaces.InspectionAttrInfo):
    is_attribute: bool = ...
    extension_type: Any = ...
    target_collection: Any = ...
    value_attr: Any = ...
    creator: Any = ...
    getset_factory: Any = ...
    proxy_factory: Any = ...
    proxy_bulk_set: Any = ...
    cascade_scalar_deletes: Any = ...
    key: Any = ...
    info: Any = ...
    def __init__(
        self,
        target_collection: Any,
        attr: Any,
        creator: Optional[Any] = ...,
        getset_factory: Optional[Any] = ...,
        proxy_factory: Optional[Any] = ...,
        proxy_bulk_set: Optional[Any] = ...,
        info: Optional[Any] = ...,
        cascade_scalar_deletes: bool = ...,
    ) -> None: ...
    def __get__(self, obj: Any, class_: Any): ...
    def __set__(self, obj: Any, values: Any): ...
    def __delete__(self, obj: Any): ...
    def for_class(self, class_: Any, obj: Optional[Any] = ...): ...

class AssociationProxyInstance:
    parent: Any = ...
    key: Any = ...
    owning_class: Any = ...
    target_collection: Any = ...
    collection_class: Any = ...
    target_class: Any = ...
    value_attr: Any = ...
    def __init__(
        self,
        parent: Any,
        owning_class: Any,
        target_class: Any,
        value_attr: Any,
    ) -> None: ...
    @classmethod
    def for_proxy(
        cls, parent: Any, owning_class: Any, parent_instance: Any
    ): ...
    def __clause_element__(self) -> None: ...
    @property
    def remote_attr(self): ...
    @property
    def local_attr(self): ...
    @property
    def attr(self): ...
    def scalar(self): ...
    @property
    def info(self): ...
    def get(self, obj: Any): ...
    def set(self, obj: Any, values: Any) -> None: ...
    def delete(self, obj: Any) -> None: ...
    def any(self, criterion: Optional[Any] = ..., **kwargs: Any): ...
    def has(self, criterion: Optional[Any] = ..., **kwargs: Any): ...

class AmbiguousAssociationProxyInstance(AssociationProxyInstance):
    def get(self, obj: Any): ...
    def __eq__(self, obj: Any) -> Any: ...
    def __ne__(self, obj: Any) -> Any: ...
    def any(self, criterion: Optional[Any] = ..., **kwargs: Any) -> None: ...
    def has(self, criterion: Optional[Any] = ..., **kwargs: Any) -> None: ...

class ObjectAssociationProxyInstance(AssociationProxyInstance):
    def contains(self, obj: Any): ...
    def __eq__(self, obj: Any) -> Any: ...
    def __ne__(self, obj: Any) -> Any: ...

class ColumnAssociationProxyInstance(
    ColumnOperators, AssociationProxyInstance
):
    def __eq__(self, other: Any) -> Any: ...
    def operate(self, op: Any, *other: Any, **kwargs: Any): ...

class _lazy_collection:
    parent: Any = ...
    target: Any = ...
    def __init__(self, obj: Any, target: Any) -> None: ...
    def __call__(self): ...

class _AssociationCollection:
    lazy_collection: Any = ...
    creator: Any = ...
    getter: Any = ...
    setter: Any = ...
    parent: Any = ...
    def __init__(
        self,
        lazy_collection: Any,
        creator: Any,
        getter: Any,
        setter: Any,
        parent: Any,
    ) -> None: ...
    col: Any = ...
    def __len__(self): ...
    def __bool__(self): ...
    __nonzero__: Any = ...

class _AssociationList(_AssociationCollection):
    def __getitem__(self, index: Any): ...
    def __setitem__(self, index: Any, value: Any) -> None: ...
    def __delitem__(self, index: Any) -> None: ...
    def __contains__(self, value: Any): ...
    def __getslice__(self, start: Any, end: Any): ...
    def __setslice__(self, start: Any, end: Any, values: Any) -> None: ...
    def __delslice__(self, start: Any, end: Any) -> None: ...
    def __iter__(self) -> Any: ...
    def append(self, value: Any) -> None: ...
    def count(self, value: Any): ...
    def extend(self, values: Any) -> None: ...
    def insert(self, index: Any, value: Any) -> None: ...
    def pop(self, index: int = ...): ...
    def remove(self, value: Any) -> None: ...
    def reverse(self) -> None: ...
    def sort(self) -> None: ...
    def clear(self) -> None: ...
    def __eq__(self, other: Any) -> Any: ...
    def __ne__(self, other: Any) -> Any: ...
    def __lt__(self, other: Any) -> Any: ...
    def __le__(self, other: Any) -> Any: ...
    def __gt__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> Any: ...
    def __cmp__(self, other: Any): ...
    def __add__(self, iterable: Any): ...
    def __radd__(self, iterable: Any): ...
    def __mul__(self, n: Any): ...
    __rmul__: Any = ...
    def __iadd__(self, iterable: Any): ...
    def __imul__(self, n: Any): ...
    def index(self, item: Any, *args: Any): ...
    def copy(self): ...
    def __hash__(self) -> Any: ...

class _AssociationDict(_AssociationCollection):
    def __getitem__(self, key: Any): ...
    def __setitem__(self, key: Any, value: Any) -> None: ...
    def __delitem__(self, key: Any) -> None: ...
    def __contains__(self, key: Any): ...
    def has_key(self, key: Any): ...
    def __iter__(self) -> Any: ...
    def clear(self) -> None: ...
    def __eq__(self, other: Any) -> Any: ...
    def __ne__(self, other: Any) -> Any: ...
    def __lt__(self, other: Any) -> Any: ...
    def __le__(self, other: Any) -> Any: ...
    def __gt__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> Any: ...
    def __cmp__(self, other: Any): ...
    def get(self, key: Any, default: Optional[Any] = ...): ...
    def setdefault(self, key: Any, default: Optional[Any] = ...): ...
    def keys(self): ...
    def iteritems(self): ...
    def itervalues(self): ...
    def iterkeys(self): ...
    def values(self): ...
    def items(self): ...
    def items(self): ...
    def values(self): ...
    def pop(self, key: Any, default: Any = ...): ...
    def popitem(self): ...
    def update(self, *a: Any, **kw: Any) -> None: ...
    def copy(self): ...
    def __hash__(self) -> Any: ...

class _AssociationSet(_AssociationCollection):
    def __len__(self): ...
    def __bool__(self): ...
    __nonzero__: Any = ...
    def __contains__(self, value: Any): ...
    def __iter__(self) -> Any: ...
    def add(self, value: Any) -> None: ...
    def discard(self, value: Any) -> None: ...
    def remove(self, value: Any) -> None: ...
    def pop(self): ...
    def update(self, other: Any) -> None: ...
    def __ior__(self, other: Any): ...
    def union(self, other: Any): ...
    __or__: Any = ...
    def difference(self, other: Any): ...
    __sub__: Any = ...
    def difference_update(self, other: Any) -> None: ...
    def __isub__(self, other: Any): ...
    def intersection(self, other: Any): ...
    __and__: Any = ...
    def intersection_update(self, other: Any) -> None: ...
    def __iand__(self, other: Any): ...
    def symmetric_difference(self, other: Any): ...
    __xor__: Any = ...
    def symmetric_difference_update(self, other: Any) -> None: ...
    def __ixor__(self, other: Any): ...
    def issubset(self, other: Any): ...
    def issuperset(self, other: Any): ...
    def clear(self) -> None: ...
    def copy(self): ...
    def __eq__(self, other: Any) -> Any: ...
    def __ne__(self, other: Any) -> Any: ...
    def __lt__(self, other: Any) -> Any: ...
    def __le__(self, other: Any) -> Any: ...
    def __gt__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> Any: ...
    def __hash__(self) -> Any: ...



from typing import Any
from typing import Optional

from .base import ENUM as ENUM
from .base import PGCompiler as PGCompiler
from .base import PGDialect as PGDialect
from .base import PGExecutionContext as PGExecutionContext
from .base import PGIdentifierPreparer as PGIdentifierPreparer
from .base import UUID as UUID
from .hstore import HSTORE as HSTORE
from .json import JSON as JSON
from .json import JSONB as JSONB
from ... import exc as exc
from ... import processors as processors
from ... import types as sqltypes
from ... import util as util
from ...util import collections_abc as collections_abc

logger: Any

class _PGNumeric(sqltypes.Numeric):
    def bind_processor(self, dialect: Any) -> None: ...
    def result_processor(self, dialect: Any, coltype: Any): ...

class _PGEnum(ENUM):
    def result_processor(self, dialect: Any, coltype: Any): ...

class _PGHStore(HSTORE):
    def bind_processor(self, dialect: Any): ...
    def result_processor(self, dialect: Any, coltype: Any): ...

class _PGJSON(JSON):
    def result_processor(self, dialect: Any, coltype: Any) -> None: ...

class _PGJSONB(JSONB):
    def result_processor(self, dialect: Any, coltype: Any) -> None: ...

class _PGUUID(UUID):
    def bind_processor(self, dialect: Any): ...
    def result_processor(self, dialect: Any, coltype: Any): ...

class PGExecutionContext_psycopg2(PGExecutionContext):
    def create_server_side_cursor(self): ...
    cursor_fetch_strategy: Any = ...
    def post_exec(self) -> None: ...

class PGCompiler_psycopg2(PGCompiler): ...
class PGIdentifierPreparer_psycopg2(PGIdentifierPreparer): ...

EXECUTEMANY_PLAIN: Any
EXECUTEMANY_BATCH: Any
EXECUTEMANY_VALUES: Any
EXECUTEMANY_VALUES_PLUS_BATCH: Any

class PGDialect_psycopg2(PGDialect):
    driver: str = ...
    supports_unicode_statements: bool = ...
    supports_server_side_cursors: bool = ...
    default_paramstyle: str = ...
    supports_sane_multi_rowcount: bool = ...
    execution_ctx_cls: Any = ...
    statement_compiler: Any = ...
    preparer: Any = ...
    psycopg2_version: Any = ...
    engine_config_types: Any = ...
    colspecs: Any = ...
    use_native_unicode: Any = ...
    use_native_hstore: Any = ...
    use_native_uuid: Any = ...
    supports_unicode_binds: Any = ...
    client_encoding: Any = ...
    executemany_mode: Any = ...
    insert_executemany_returning: bool = ...
    executemany_batch_page_size: Any = ...
    executemany_values_page_size: Any = ...
    def __init__(
        self,
        use_native_unicode: bool = ...,
        client_encoding: Optional[Any] = ...,
        use_native_hstore: bool = ...,
        use_native_uuid: bool = ...,
        executemany_mode: str = ...,
        executemany_batch_page_size: int = ...,
        executemany_values_page_size: int = ...,
        **kwargs: Any,
    ) -> None: ...
    def initialize(self, connection: Any) -> None: ...
    @classmethod
    def dbapi(cls): ...
    def set_isolation_level(self, connection: Any, level: Any) -> None: ...
    def set_readonly(self, connection: Any, value: Any) -> None: ...
    def get_readonly(self, connection: Any): ...
    def set_deferrable(self, connection: Any, value: Any) -> None: ...
    def get_deferrable(self, connection: Any): ...
    def on_connect(self): ...
    def do_executemany(
        self,
        cursor: Any,
        statement: Any,
        parameters: Any,
        context: Optional[Any] = ...,
    ) -> None: ...
    def create_connect_args(self, url: Any): ...
    def is_disconnect(self, e: Any, connection: Any, cursor: Any): ...

dialect = PGDialect_psycopg2



from typing import Callable
from typing import List
from typing import Optional
from typing import Sequence
from typing import TypeVar

_T = TypeVar("_T")
OrderingFunc = Callable[[int, Sequence[_T]], int]

def ordering_list(
    attr: str,
    count_from: Optional[int] = ...,
    ordering_func: Optional[OrderingFunc] = ...,
    reorder_on_append: bool = ...,
) -> Callable[[], OrderingList]: ...

class OrderingList(List[_T]):
    ordering_attr: str = ...
    ordering_func: OrderingFunc = ...
    reorder_on_append: bool = ...
    def __init__(
        self,
        ordering_attr: Optional[str] = ...,
        ordering_func: Optional[OrderingFunc] = ...,
        reorder_on_append: bool = ...,
    ) -> None: ...
    def reorder(self) -> None: ...



from typing import Any

from .base import _ConnDialect
from .base import Pool as Pool
from .. import exc as exc
from .. import util as util
from ..util import chop_traceback as chop_traceback
from ..util import threading as threading

class QueuePool(Pool):
    def __init__(
        self,
        creator: Any,
        pool_size: int = ...,
        max_overflow: int = ...,
        timeout: float = ...,
        use_lifo: bool = ...,
        **kw: Any,
    ) -> None: ...
    def recreate(self): ...
    def dispose(self) -> None: ...
    def status(self): ...
    def size(self): ...
    def timeout(self): ...
    def checkedin(self): ...
    def overflow(self): ...
    def checkedout(self): ...

class _AsyncConnDialect(_ConnDialect):
    is_async: bool = ...

class AsyncAdaptedQueuePool(QueuePool): ...
class FallbackAsyncAdaptedQueuePool(AsyncAdaptedQueuePool): ...

class NullPool(Pool):
    def status(self): ...
    def recreate(self): ...
    def dispose(self) -> None: ...

class SingletonThreadPool(Pool):
    size: Any = ...
    def __init__(
        self, creator: Any, pool_size: int = ..., **kw: Any
    ) -> None: ...
    def recreate(self): ...
    def dispose(self) -> None: ...
    def status(self): ...
    def connect(self): ...

class StaticPool(Pool):
    def connection(self): ...
    def status(self): ...
    def dispose(self) -> None: ...
    def recreate(self): ...

class AssertionPool(Pool):
    def __init__(self, *args: Any, **kw: Any) -> None: ...
    def status(self): ...
    def dispose(self) -> None: ...
    def recreate(self): ...



from typing import Any

from .base import BFILE as BFILE
from .base import BINARY_DOUBLE as BINARY_DOUBLE
from .base import BINARY_FLOAT as BINARY_FLOAT
from .base import BLOB as BLOB
from .base import CHAR as CHAR
from .base import CLOB as CLOB
from .base import DATE as DATE
from .base import DOUBLE_PRECISION as DOUBLE_PRECISION
from .base import FLOAT as FLOAT
from .base import INTERVAL as INTERVAL
from .base import LONG as LONG
from .base import NCHAR as NCHAR
from .base import NCLOB as NCLOB
from .base import NUMBER as NUMBER
from .base import NVARCHAR as NVARCHAR
from .base import NVARCHAR2 as NVARCHAR2
from .base import RAW as RAW
from .base import ROWID as ROWID
from .base import TIMESTAMP as TIMESTAMP
from .base import VARCHAR as VARCHAR
from .base import VARCHAR2 as VARCHAR2

dialect: Any



from typing import Any

from .pysqlite import SQLiteDialect_pysqlite as SQLiteDialect_pysqlite
from ... import pool as pool

class SQLiteDialect_pysqlcipher(SQLiteDialect_pysqlite):
    driver: str = ...
    pragmas: Any = ...
    @classmethod
    def dbapi(cls): ...
    @classmethod
    def get_pool_class(cls, url: Any): ...
    def connect(self, *cargs: Any, **cparams: Any): ...
    def create_connect_args(self, url: Any): ...

dialect = SQLiteDialect_pysqlcipher



from typing import Any
from typing import Callable
from typing import Iterator
from typing import Optional
from typing import Type

from . import elements
from . import roles
from .base import Options
from .operators import ColumnOperators

class LambdaOptions(Options):
    enable_tracking: bool = ...
    track_closure_variables: bool = ...
    track_on: Any = ...
    global_track_bound_values: bool = ...
    track_bound_values: bool = ...
    lambda_cache: Any = ...

def lambda_stmt(
    lmb: Any,
    enable_tracking: bool = ...,
    track_closure_variables: bool = ...,
    track_on: Optional[Any] = ...,
    global_track_bound_values: bool = ...,
    track_bound_values: bool = ...,
    lambda_cache: Optional[Any] = ...,
) -> StatementLambdaElement: ...

class LambdaElement(elements.ClauseElement):
    __visit_name__: str = ...
    parent_lambda: Any = ...
    fn: Callable[..., elements.ClauseElement] = ...
    role: Type[Any] = ...
    tracker_key: Any = ...
    opts: Any = ...
    def __init__(
        self,
        fn: Callable[..., elements.ClauseElement],
        role: Type[Any],
        opts: Any = ...,
        apply_propagate_attrs: Optional[Any] = ...,
    ) -> None: ...
    def __getattr__(self, key: Any) -> Any: ...

class DeferredLambdaElement(LambdaElement):
    lambda_args: Any = ...
    def __init__(
        self,
        fn: Callable[..., elements.ClauseElement],
        role: Type[Any],
        opts: Any = ...,
        lambda_args: Any = ...,
    ) -> None: ...

class StatementLambdaElement(roles.AllowsLambdaRole, LambdaElement):
    def __add__(self, other: Any) -> LinkedLambdaElement: ...
    def add_criteria(
        self,
        other: Any,
        enable_tracking: bool = ...,
        track_on: Optional[Any] = ...,
        track_closure_variables: bool = ...,
        track_bound_values: bool = ...,
    ) -> LinkedLambdaElement: ...
    def spoil(self) -> NullLambdaStatement: ...

class NullLambdaStatement(roles.AllowsLambdaRole, elements.ClauseElement):
    __visit_name__: str = ...
    def __init__(self, statement: Any) -> None: ...
    def __getattr__(self, key: Any) -> Any: ...
    def __add__(self, other: Any) -> NullLambdaStatement: ...
    def add_criteria(self, other: Any, **kw: Any) -> NullLambdaStatement: ...

class LinkedLambdaElement(StatementLambdaElement):
    role: Any = ...
    opts: Any = ...
    fn: Any = ...
    parent_lambda: Any = ...
    tracker_key: Any = ...
    def __init__(
        self,
        fn: Callable[..., elements.ClauseElement],
        parent_lambda: StatementLambdaElement,
        opts: Any,
    ) -> None: ...

class AnalyzedCode:
    @classmethod
    def get(
        cls,
        fn: Callable[..., elements.ClauseElement],
        lambda_element: LambdaElement,
        lambda_kw: Any,
        **kw: Any,
    ) -> Any: ...
    track_bound_values: Any = ...
    track_closure_variables: Any = ...
    bindparam_trackers: Any = ...
    closure_trackers: Any = ...
    build_py_wrappers: Any = ...
    def __init__(
        self,
        fn: Callable[..., elements.ClauseElement],
        lambda_element: LambdaElement,
        opts: Any,
    ) -> None: ...

class AnalyzedFunction:
    analyzed_code: Any = ...
    fn: Callable[..., elements.ClauseElement] = ...
    bindparam_trackers: Any = ...
    def __init__(
        self,
        analyzed_code: Any,
        lambda_element: LambdaElement,
        apply_propagate_attrs: Any,
        fn: Callable[..., elements.ClauseElement],
    ) -> None: ...

class PyWrapper(ColumnOperators):
    fn: Callable[..., elements.ClauseElement] = ...
    track_bound_values: Any = ...
    def __init__(
        self,
        fn: Callable[..., elements.ClauseElement],
        name: str,
        to_evaluate: Any,
        closure_index: Optional[Any] = ...,
        getter: Optional[Any] = ...,
        track_bound_values: bool = ...,
    ) -> None: ...
    def __call__(self, *arg: Any, **kw: Any) -> Any: ...
    def operate(
        self, op: Any, *other: Any, **kwargs: Any
    ) -> elements.ClauseElement: ...
    def reverse_operate(
        self, op: Any, other: Any, **kwargs: Any
    ) -> elements.ClauseElement: ...
    def __clause_element__(self) -> elements.ClauseElement: ...
    def __bool__(self) -> bool: ...
    def __nonzero__(self) -> bool: ...
    def __getattribute__(self, key: Any) -> Any: ...
    def __iter__(self) -> Iterator[Any]: ...
    def __getitem__(self, key: Any) -> Any: ...

def insp(lmb: Any) -> Any: ...



from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import TypeVar
from typing import Union

from typing_extensions import Literal
from typing_extensions import Protocol

from .cursor import CursorResult
from .interfaces import _DBAPIConnection
from .interfaces import _DBAPICursor
from .interfaces import Connectable as Connectable
from .interfaces import Dialect
from .interfaces import ExceptionContext
from .interfaces import ExecutionContext
from .url import URL
from .. import log
from .. import util
from ..exc import StatementError
from ..pool import Pool

_T = TypeVar("_T")
_T_co = TypeVar("_T_co", covariant=True)
_T_contra = TypeVar("_T_contra", contravariant=True)
_TConnection = TypeVar("_TConnection", bound=Connection)
_TTransaction = TypeVar("_TTransaction", bound=Transaction)
_TEngine = TypeVar("_TEngine", bound=Engine)

_ExecutionOptions: util.immutabledict[Any, Any]

class _ConnectionCallable(Protocol[_T_contra, _T_co]):
    def __call__(
        self, __connection: _T_contra, *args: Any, **kwargs: Any
    ) -> _T_co: ...

class Connection(Connectable):
    engine: Engine = ...
    dialect: Dialect = ...
    should_close_with_result: bool = ...
    dispatch: Any = ...
    def __init__(
        self,
        engine: Engine,
        connection: Optional[_DBAPIConnection] = ...,
        close_with_result: bool = ...,
        _branch_from: Optional[Any] = ...,
        _execution_options: Optional[Dict[str, Any]] = ...,
        _dispatch: Optional[Any] = ...,
        _has_events: Optional[Any] = ...,
    ) -> None: ...
    def schema_for_object(self, obj: Any) -> str: ...
    def __enter__(self: _TConnection) -> _TConnection: ...
    def __exit__(self, type_: Any, value: Any, traceback: Any) -> None: ...
    def execution_options(self: _TConnection, **opt: Any) -> _TConnection: ...
    def get_execution_options(self) -> Dict[str, Any]: ...
    @property
    def closed(self) -> bool: ...
    @property
    def invalidated(self) -> bool: ...
    @property
    def connection(self) -> _DBAPIConnection: ...
    def get_isolation_level(self) -> Any: ...
    @property
    def default_isolation_level(self) -> Any: ...
    @property
    def info(self) -> Dict[Any, Any]: ...
    def connect(self: _TConnection, close_with_result: bool = ...) -> _TConnection: ...  # type: ignore[override]
    def invalidate(self, exception: Optional[Any] = ...) -> None: ...
    def detach(self) -> None: ...
    def begin(self) -> Optional[Transaction]: ...
    def begin_nested(self) -> NestedTransaction: ...
    def begin_twophase(
        self, xid: Optional[str] = ...
    ) -> TwoPhaseTransaction: ...
    def recover_twophase(self) -> None: ...
    def rollback_prepared(self, xid: str, recover: bool = ...) -> None: ...
    def commit_prepared(self, xid: str, recover: bool = ...) -> None: ...
    def in_transaction(self) -> bool: ...
    def in_nested_transaction(self) -> bool: ...
    def get_transaction(self) -> Optional[Transaction]: ...
    def get_nested_transaction(self) -> Optional[NestedTransaction]: ...
    def close(self) -> None: ...
    def scalar(
        self, object_: Any, *multiparams: Any, **params: Any
    ) -> Any: ...
    def execute(self, statement: Any, *multiparams: Any, **params: Any) -> CursorResult: ...  # type: ignore[override]
    def exec_driver_sql(
        self,
        statement: str,
        parameters: Optional[Any] = ...,
        execution_options: Optional[Any] = ...,
    ) -> CursorResult: ...
    def transaction(
        self: _TConnection,
        callable_: _ConnectionCallable[_TConnection, _T],
        *args: Any,
        **kwargs: Any,
    ) -> _T: ...
    def run_callable(
        self: _TConnection,
        callable_: _ConnectionCallable[_TConnection, _T],
        *args: Any,
        **kwargs: Any,
    ) -> _T: ...

class ExceptionContextImpl(ExceptionContext):
    engine: Engine = ...
    connection: Connection = ...
    sqlalchemy_exception: Optional[StatementError] = ...
    original_exception: BaseException = ...
    execution_context: Optional[ExecutionContext] = ...
    statement: Optional[str] = ...
    parameters: Optional[Any] = ...
    is_disconnect: bool = ...
    invalidate_pool_on_disconnect: bool = ...
    def __init__(
        self,
        exception: BaseException,
        sqlalchemy_exception: Optional[StatementError],
        engine: Engine,
        connection: Optional[Connection],
        cursor: Optional[_DBAPICursor],
        statement: Optional[str],
        parameters: Optional[Any],
        context: Optional[ExecutionContext],
        is_disconnect: bool,
        invalidate_pool_on_disconnect: bool,
    ) -> None: ...

class Transaction:
    def __init__(self, connection: Connection) -> None: ...
    @property
    def is_valid(self) -> bool: ...
    def close(self) -> None: ...
    def rollback(self) -> None: ...
    def commit(self) -> None: ...
    def __enter__(self: _TTransaction) -> _TTransaction: ...
    def __exit__(self, type_: Any, value: Any, traceback: Any) -> None: ...

class MarkerTransaction(Transaction):
    connection: Connection = ...
    def __init__(self, connection: Connection) -> None: ...
    @property
    def is_active(self) -> bool: ...

class RootTransaction(Transaction):
    connection: Connection = ...
    is_active: bool = ...
    def __init__(self, connection: Connection) -> None: ...

class NestedTransaction(Transaction):
    connection: Connection = ...
    is_active: bool = ...
    def __init__(self, connection: Connection) -> None: ...

class TwoPhaseTransaction(RootTransaction):
    xid: str = ...
    def __init__(self, connection: Connection, xid: str) -> None: ...
    def prepare(self) -> None: ...

class Engine(Connectable, log.Identified):
    pool: Pool = ...
    url: URL = ...
    dialect: Dialect = ...
    logging_name: Optional[str] = ...
    echo: Optional[Union[bool, Literal["debug"]]] = ...
    hide_parameters: bool = ...
    def __init__(
        self,
        pool: Pool,
        dialect: Dialect,
        url: URL,
        logging_name: Optional[str] = ...,
        echo: Optional[Union[bool, Literal["debug"]]] = ...,
        query_cache_size: int = ...,
        execution_options: Optional[Dict[str, Any]] = ...,
        hide_parameters: bool = ...,
    ) -> None: ...
    @property
    def engine(self: _TEngine) -> _TEngine: ...  # type: ignore[override]
    def clear_compiled_cache(self) -> None: ...
    def update_execution_options(self, **opt: Any) -> None: ...
    def execution_options(self, **opt: Any) -> OptionEngine: ...
    def get_execution_options(self) -> Dict[str, Any]: ...
    @property
    def name(self) -> str: ...
    @property
    def driver(self) -> str: ...
    def dispose(self) -> None: ...
    class _trans_ctx:
        conn: Connection = ...
        transaction: Transaction = ...
        close_with_result: bool = ...
        def __init__(
            self,
            conn: Connection,
            transaction: Transaction,
            close_with_result: bool,
        ) -> None: ...
        def __enter__(self) -> Connection: ...
        def __exit__(self, type_: Any, value: Any, traceback: Any) -> None: ...
    def begin(self, close_with_result: bool = ...) -> _trans_ctx: ...
    def transaction(
        self,
        callable_: _ConnectionCallable[_TConnection, _T],
        *args: Any,
        **kwargs: Any,
    ) -> _T: ...
    def run_callable(
        self,
        callable_: _ConnectionCallable[_TConnection, _T],
        *args: Any,
        **kwargs: Any,
    ) -> _T: ...
    def execute(  # type: ignore[override]
        self, statement: Any, *multiparams: Any, **params: Any
    ) -> CursorResult: ...
    def scalar(  # type: ignore[override]
        self, statement: Any, *multiparams: Any, **params: Any
    ) -> Any: ...
    def connect(self, close_with_result: bool = ...) -> Connection: ...  # type: ignore[override]
    def table_names(
        self,
        schema: Optional[str] = ...,
        connection: Optional[Connection] = ...,
    ) -> List[str]: ...
    def has_table(
        self, table_name: str, schema: Optional[str] = ...
    ) -> bool: ...
    def raw_connection(
        self, _connection: Optional[Connection] = ...
    ) -> _DBAPIConnection: ...

class OptionEngineMixin:
    url: URL = ...
    dialect: Dialect = ...
    logging_name: Optional[str] = ...
    echo: Optional[Union[bool, Literal["debug"]]] = ...
    hide_parameters: bool = ...
    dispatch: Any = ...
    pool: Pool = ...
    def __init__(
        self, proxied: Engine, execution_options: Dict[str, Any]
    ) -> None: ...

class OptionEngine(OptionEngineMixin, Engine): ...



from typing import Any

from mypy.plugin import Callable as Callable
from mypy.plugin import ClassDefContext as ClassDefContext
from mypy.plugin import Optional as Optional
from mypy.plugin import Plugin

prefix: str

class CustomPlugin(Plugin):
    def get_class_decorator_hook(
        self, fullname: str
    ) -> Optional[Callable[[ClassDefContext], None]]: ...
    def get_customize_class_mro_hook(
        self, fullname: str
    ) -> Optional[Callable[[ClassDefContext], None]]: ...

def fill_in_decorators(ctx: ClassDefContext) -> None: ...
def cls_decorator_hook(ctx: ClassDefContext) -> None: ...

COLUMN_THINGY: int
RELATIONSHIP_THINGY: int

def plugin(version: str) -> Any: ...



from typing import Any

from . import base as base
from . import interfaces as interfaces
from .base import ATTR_WAS_SET as ATTR_WAS_SET
from .base import INIT_OK as INIT_OK
from .base import NEVER_SET as NEVER_SET
from .base import NO_VALUE as NO_VALUE
from .base import PASSIVE_NO_INITIALIZE as PASSIVE_NO_INITIALIZE
from .base import PASSIVE_NO_RESULT as PASSIVE_NO_RESULT
from .base import PASSIVE_OFF as PASSIVE_OFF
from .base import SQL_OK as SQL_OK
from .path_registry import PathRegistry as PathRegistry
from .. import inspection as inspection
from .. import util as util

class InstanceState(interfaces.InspectionAttrInfo):
    session_id: Any = ...
    key: Any = ...
    runid: Any = ...
    load_options: Any = ...
    load_path: Any = ...
    insert_order: Any = ...
    modified: bool = ...
    expired: bool = ...
    is_instance: bool = ...
    identity_token: Any = ...
    callables: Any = ...
    class_: Any = ...
    manager: Any = ...
    committed_state: Any = ...
    expired_attributes: Any = ...
    def __init__(self, obj: Any, manager: Any) -> None: ...
    def attrs(self): ...
    @property
    def transient(self): ...
    @property
    def pending(self): ...
    @property
    def deleted(self): ...
    @property
    def was_deleted(self): ...
    @property
    def persistent(self): ...
    @property
    def detached(self): ...
    @property
    def session(self): ...
    @property
    def object(self): ...
    @property
    def identity(self): ...
    @property
    def identity_key(self): ...
    def parents(self): ...
    def mapper(self): ...
    @property
    def has_identity(self): ...
    def obj(self) -> None: ...
    @property
    def dict(self): ...
    def get_history(self, key: Any, passive: Any): ...
    def get_impl(self, key: Any): ...
    @property
    def unmodified(self): ...
    def unmodified_intersection(self, keys: Any): ...
    @property
    def unloaded(self): ...
    @property
    def unloaded_expirable(self): ...

class AttributeState:
    state: Any = ...
    key: Any = ...
    def __init__(self, state: Any, key: Any) -> None: ...
    @property
    def loaded_value(self): ...
    @property
    def value(self): ...
    @property
    def history(self): ...
    def load_history(self): ...

class PendingCollection:
    deleted_items: Any = ...
    added_items: Any = ...
    def __init__(self) -> None: ...
    def append(self, value: Any) -> None: ...
    def remove(self, value: Any) -> None: ...
